from collections import OrderedDict
from typing import Optional, Tuple

from ..en import Provider as AddressProvider


class Provider(AddressProvider):
    city_prefixes = ("North", "East", "West", "South", "New", "Lake", "Port")

    city_suffixes = (
        "town",
        "ton",
        "land",
        "ville",
        "berg",
        "burgh",
        "borough",
        "bury",
        "view",
        "port",
        "mouth",
        "stad",
        "furt",
        "chester",
        "mouth",
        "fort",
        "haven",
        "side",
        "shire",
    )

    building_number_formats = ("#####", "####", "###")

    street_suffixes = (
        "Alley",
        "Avenue",
        "Branch",
        "Bridge",
        "Brook",
        "Brooks",
        "Burg",
        "Burgs",
        "Bypass",
        "Camp",
        "Canyon",
        "Cape",
        "Causeway",
        "Center",
        "Centers",
        "Circle",
        "Circles",
        "Cliff",
        "Cliffs",
        "Club",
        "Common",
        "Corner",
        "Corners",
        "Course",
        "Court",
        "Courts",
        "Cove",
        "Coves",
        "Creek",
        "Crescent",
        "Crest",
        "Crossing",
        "Crossroad",
        "Curve",
        "Dale",
        "Dam",
        "Divide",
        "Drive",
        "Drive",
        "Drives",
        "Estate",
        "Estates",
        "Expressway",
        "Extension",
        "Extensions",
        "Fall",
        "Falls",
        "Ferry",
        "Field",
        "Fields",
        "Flat",
        "Flats",
        "Ford",
        "Fords",
        "Forest",
        "Forge",
        "Forges",
        "Fork",
        "Forks",
        "Fort",
        "Freeway",
        "Garden",
        "Gardens",
        "Gateway",
        "Glen",
        "Glens",
        "Green",
        "Greens",
        "Grove",
        "Groves",
        "Harbor",
        "Harbors",
        "Haven",
        "Heights",
        "Highway",
        "Hill",
        "Hills",
        "Hollow",
        "Inlet",
        "Inlet",
        "Island",
        "Island",
        "Islands",
        "Islands",
        "Isle",
        "Isle",
        "Junction",
        "Junctions",
        "Key",
        "Keys",
        "Knoll",
        "Knolls",
        "Lake",
        "Lakes",
        "Land",
        "Landing",
        "Lane",
        "Light",
        "Lights",
        "Loaf",
        "Lock",
        "Locks",
        "Locks",
        "Lodge",
        "Lodge",
        "Loop",
        "Mall",
        "Manor",
        "Manors",
        "Meadow",
        "Meadows",
        "Mews",
        "Mill",
        "Mills",
        "Mission",
        "Mission",
        "Motorway",
        "Mount",
        "Mountain",
        "Mountain",
        "Mountains",
        "Mountains",
        "Neck",
        "Orchard",
        "Oval",
        "Overpass",
        "Park",
        "Parks",
        "Parkway",
        "Parkways",
        "Pass",
        "Passage",
        "Path",
        "Pike",
        "Pine",
        "Pines",
        "Place",
        "Plain",
        "Plains",
        "Plains",
        "Plaza",
        "Plaza",
        "Point",
        "Points",
        "Port",
        "Port",
        "Ports",
        "Ports",
        "Prairie",
        "Prairie",
        "Radial",
        "Ramp",
        "Ranch",
        "Rapid",
        "Rapids",
        "Rest",
        "Ridge",
        "Ridges",
        "River",
        "Road",
        "Road",
        "Roads",
        "Roads",
        "Route",
        "Row",
        "Rue",
        "Run",
        "Shoal",
        "Shoals",
        "Shore",
        "Shores",
        "Skyway",
        "Spring",
        "Springs",
        "Springs",
        "Spur",
        "Spurs",
        "Square",
        "Square",
        "Squares",
        "Squares",
        "Station",
        "Station",
        "Stravenue",
        "Stravenue",
        "Stream",
        "Stream",
        "Street",
        "Street",
        "Streets",
        "Summit",
        "Summit",
        "Terrace",
        "Throughway",
        "Trace",
        "Track",
        "Trafficway",
        "Trail",
        "Trail",
        "Tunnel",
        "Tunnel",
        "Turnpike",
        "Turnpike",
        "Underpass",
        "Union",
        "Unions",
        "Valley",
        "Valleys",
        "Via",
        "Viaduct",
        "View",
        "Views",
        "Village",
        "Village",
        "Villages",
        "Ville",
        "Vista",
        "Vista",
        "Walk",
        "Walks",
        "Wall",
        "Way",
        "Ways",
        "Well",
        "Wells",
    )

    postcode_formats = ("#####", "#####-####")

    states = (
        "Alabama",
        "Alaska",
        "Arizona",
        "Arkansas",
        "California",
        "Colorado",
        "Connecticut",
        "Delaware",
        "Florida",
        "Georgia",
        "Hawaii",
        "Idaho",
        "Illinois",
        "Indiana",
        "Iowa",
        "Kansas",
        "Kentucky",
        "Louisiana",
        "Maine",
        "Maryland",
        "Massachusetts",
        "Michigan",
        "Minnesota",
        "Mississippi",
        "Missouri",
        "Montana",
        "Nebraska",
        "Nevada",
        "New Hampshire",
        "New Jersey",
        "New Mexico",
        "New York",
        "North Carolina",
        "North Dakota",
        "Ohio",
        "Oklahoma",
        "Oregon",
        "Pennsylvania",
        "Rhode Island",
        "South Carolina",
        "South Dakota",
        "Tennessee",
        "Texas",
        "Utah",
        "Vermont",
        "Virginia",
        "Washington",
        "West Virginia",
        "Wisconsin",
        "Wyoming",
    )
    states_abbr = (
        "AL",
        "AK",
        "AZ",
        "AR",
        "CA",
        "CO",
        "CT",
        "DE",
        "DC",
        "FL",
        "GA",
        "HI",
        "ID",
        "IL",
        "IN",
        "IA",
        "KS",
        "KY",
        "LA",
        "ME",
        "MD",
        "MA",
        "MI",
        "MN",
        "MS",
        "MO",
        "MT",
        "NE",
        "NV",
        "NH",
        "NJ",
        "NM",
        "NY",
        "NC",
        "ND",
        "OH",
        "OK",
        "OR",
        "PA",
        "RI",
        "SC",
        "SD",
        "TN",
        "TX",
        "UT",
        "VT",
        "VA",
        "WA",
        "WV",
        "WI",
        "WY",
    )

    states_postcode = {
        "AL": (35004, 36925),
        "AK": (99501, 99950),
        "AZ": (85001, 86556),
        "AR": (71601, 72959),
        "CA": (90001, 96162),
        "CO": (80001, 81658),
        "CT": (6001, 6389),
        "DE": (19701, 19980),
        "DC": (20001, 20039),
        "FL": (32004, 34997),
        "GA": (30001, 31999),
        "HI": (96701, 96898),
        "ID": (83201, 83876),
        "IL": (60001, 62999),
        "IN": (46001, 47997),
        "IA": (50001, 52809),
        "KS": (66002, 67954),
        "KY": (40003, 42788),
        "LA": (70001, 71232),
        "ME": (3901, 4992),
        "MD": (20812, 21930),
        "MA": (1001, 2791),
        "MI": (48001, 49971),
        "MN": (55001, 56763),
        "MS": (38601, 39776),
        "MO": (63001, 65899),
        "MT": (59001, 59937),
        "NE": (68001, 68118),
        "NV": (88901, 89883),
        "NH": (3031, 3897),
        "NJ": (7001, 8989),
        "NM": (87001, 88441),
        "NY": (10001, 14905),
        "NC": (27006, 28909),
        "ND": (58001, 58856),
        "OH": (43001, 45999),
        "OK": (73001, 73199),
        "OR": (97001, 97920),
        "PA": (15001, 19640),
        "RI": (2801, 2940),
        "SC": (29001, 29948),
        "SD": (57001, 57799),
        "TN": (37010, 38589),
        "TX": (75503, 79999),
        "UT": (84001, 84784),
        "VT": (5001, 5495),
        "VA": (22001, 24658),
        "WA": (98001, 99403),
        "WV": (24701, 26886),
        "WI": (53001, 54990),
        "WY": (82001, 83128),
        # Territories & freely-associated states
        # incomplete ranges with accurate subsets - https://www.geonames.org/postalcode-search.html
        "AS": (96799, 96799),
        "FM": (96941, 96944),
        "GU": (96910, 96932),
        "MH": (96960, 96970),
        "MP": (96950, 96952),
        "PW": (96940, 96940),
        "PR": (600, 799),
        "VI": (801, 805),
    }

    territories_abbr = (
        "AS",
        "GU",
        "MP",
        "PR",
        "VI",
    )

    # Freely-associated states (sovereign states; members of COFA)
    # https://en.wikipedia.org/wiki/Compact_of_Free_Association
    freely_associated_states_abbr = (
        "FM",
        "MH",
        "PW",
    )

    known_usps_abbr = states_abbr + territories_abbr + freely_associated_states_abbr

    military_state_abbr = ("AE", "AA", "AP")

    military_ship_prefix = ("USS", "USNS", "USNV", "USCGC")

    military_apo_format = "PSC ####, Box ####"

    military_dpo_format = "Unit #### Box ####"

    city_formats = (
        "{{city_prefix}} {{first_name}}{{city_suffix}}",
        "{{city_prefix}} {{first_name}}",
        "{{first_name}}{{city_suffix}}",
        "{{last_name}}{{city_suffix}}",
    )

    street_name_formats = (
        "{{first_name}} {{street_suffix}}",
        "{{last_name}} {{street_suffix}}",
    )

    street_address_formats = (
        "{{building_number}} {{street_name}}",
        "{{building_number}} {{street_name}} {{secondary_address}}",
    )

    address_formats = OrderedDict(
        (
            ("{{street_address}}\n{{city}}, {{state_abbr}} {{postcode}}", 25.0),
            #  military address formatting.
            ("{{military_apo}}\nAPO {{military_state}} {{postcode}}", 1.0),
            (
                "{{military_ship}} {{last_name}}\nFPO {{military_state}} {{postcode}}",
                1.0,
            ),
            ("{{military_dpo}}\nDPO {{military_state}} {{postcode}}", 1.0),
        )
    )

    secondary_address_formats = ("Apt. ###", "Suite ###")

    def city_prefix(self) -> str:
        return self.random_element(self.city_prefixes)

    def secondary_address(self) -> str:
        return self.numerify(self.random_element(self.secondary_address_formats))

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit

    def state_abbr(
        self,
        include_territories: bool = True,
        include_freely_associated_states: bool = True,
    ) -> str:
        """
        :returns: A random two-letter USPS postal code

        By default, the resulting code may abbreviate any of the fity states,
        five US territories, or three freely-associating sovereign states.

        :param include_territories: If True, territories will be included.
            If False, US territories will be excluded.
        :param include_freely_associated_states: If True, freely-associated states will be included.
            If False, sovereign states in free association with the US will be excluded.
        """
        abbreviations: Tuple[str, ...] = self.states_abbr
        if include_territories:
            abbreviations += self.territories_abbr
        if include_freely_associated_states:
            abbreviations += self.freely_associated_states_abbr
        return self.random_element(abbreviations)

    def postcode(self) -> str:
        return "%05d" % self.generator.random.randint(501, 99950)

    def zipcode_plus4(self) -> str:
        return "%s-%04d" % (self.zipcode(), self.generator.random.randint(1, 9999))

    def postcode_in_state(self, state_abbr: Optional[str] = None) -> str:
        """
        :returns: A random postcode within the provided state abbreviation

        :param state_abbr: A state abbreviation
        """
        if state_abbr is None:
            state_abbr = self.random_element(self.states_abbr)

        if state_abbr in self.known_usps_abbr:
            postcode = "%d" % (
                self.generator.random.randint(
                    self.states_postcode[state_abbr][0],
                    self.states_postcode[state_abbr][1],
                )
            )

            # zero left pad up until desired length (some have length 3 or 4)
            target_postcode_len = 5
            current_postcode_len = len(postcode)
            if current_postcode_len < target_postcode_len:
                pad = target_postcode_len - current_postcode_len
                postcode = f"{'0'*pad}{postcode}"

            return postcode

        else:
            raise Exception("State Abbreviation not found in list")

    def military_ship(self) -> str:
        """
        :example: 'USS'
        """
        return self.random_element(self.military_ship_prefix)

    def military_state(self) -> str:
        """
        :example: 'APO'
        """
        return self.random_element(self.military_state_abbr)

    def military_apo(self) -> str:
        """
        :example: 'PSC 5394 Box 3492
        """
        return self.numerify(self.military_apo_format)

    def military_dpo(self) -> str:
        """
        :example: 'Unit 3333 Box 9342'
        """
        return self.numerify(self.military_dpo_format)

    # Aliases
    def zipcode(self) -> str:
        return self.postcode()

    def zipcode_in_state(self, state_abbr: Optional[str] = None) -> str:
        return self.postcode_in_state(state_abbr)

    def postalcode(self) -> str:
        return self.postcode()

    def postalcode_in_state(self, state_abbr: Optional[str] = None) -> str:
        return self.postcode_in_state(state_abbr)

    def postalcode_plus4(self) -> str:
        return self.zipcode_plus4()



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ru_RU`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Russia
    - https://ru.wikipedia.org/wiki/Категории_транспортных_средств
    """

    license_plate_letters = ("A", "B", "E", "K", "M", "Н", "О", "Р", "С", "Т", "У", "Х")

    vehicle_categories = (
        "M",
        "A",
        "A1",
        "B",
        "B1",
        "BE",
        "C",
        "C1",
        "C1E",
        "CE",
        "D",
        "D1",
        "DE",
        "Tm",
        "Tb",
    )

    license_plate_suffix = (
        # Republic of Adygea
        "01",
        # Republic of Bashkortostan
        "02",
        "102",
        # Republic of Buryatia
        "03",
        # Altai Republic
        "04",
        # Republic of Dagestan
        "05",
        # Republic of Ingushetia
        "06",
        # Kabardino-Balkar Republic
        "07",
        # Republic of Kalmykia
        "08",
        # Karachay-Cherkess Republic
        "09",
        # Republic of Karelia
        "10",
        # Komi Republic
        "11",
        # Mari El Republic
        "12",
        # Republic of Mordovia
        "13",
        "113",
        # Sakha Republic
        "14",
        # Republic of North Ossetia–Alania
        "15",
        # Republic of Tatarstan
        "16",
        "116",
        "716",
        # Tuva Republic
        "17",
        # Udmurt Republic
        "18",
        # Republic of Khakassia
        "19",
        # Chechen Republic
        "20",
        "95",
        # Chuvash Republic
        "21",
        "121",
        # Altai Krai
        "22",
        # Krasnodar Krai
        "23",
        "93",
        "123",
        # Krasnoyarsk Krai
        "24",
        "84",
        "88",
        "124",
        # Primorsky Krai
        "25",
        "125",
        # Stavropol Krai
        "26",
        "126",
        # Khabarovsk Krai
        "27",
        # Amur Oblast
        "28",
        # Arkhangelsk Oblast
        "29",
        # Astrakhan Oblast
        "30",
        # Belgorod Oblast
        "31",
        # Bryansk Oblast
        "32",
        # Vladimir Oblast
        "33",
        # Volgograd Oblast
        "34",
        "134",
        # Vologda Oblast
        "35",
        # Voronezh Oblast
        "36",
        "136",
        # Ivanovo Oblast
        "37",
        # Irkutsk Oblast
        "38",
        "85",
        "38",
        # Kaliningrad Oblast
        "39",
        "91",
        # Kaluga Oblast
        "40",
        # Kamchatka Krai
        "41",
        "82",
        # Kemerovo Oblast
        "42",
        "142",
        # Kirov Oblast
        "43",
        # Kostroma Oblast
        "44",
        # Kurgan Oblast
        "45",
        # Kursk Oblast
        "46",
        # Leningrad Oblast
        "47",
        # Lipetsk Oblast
        "48",
        # Magadan Oblast
        "49",
        # Moscow Oblast
        "50",
        "90",
        "150",
        "190",
        "750",
        # Murmansk Oblast
        "51",
        # Nizhny Novgorod Oblast
        "52",
        "152",
        # Novgorod Oblast
        "53",
        # Novosibirsk Oblast
        "54",
        "154",
        # Omsk Oblast
        "55",
        # Orenburg Oblast
        "56",
        # Oryol Oblast
        "57",
        # Penza Oblast
        "58",
        # Perm Krai
        "59",
        "81",
        "159",
        # Pskov Oblast
        "60",
        # Rostov Oblast
        "61",
        "161",
        # Ryazan Oblast
        "62",
        # Samara Oblast
        "63",
        "163",
        "763",
        # Saratov Oblast
        "64",
        "164",
        # Sakhalin Oblast
        "65",
        # Sverdlovsk Oblast
        "66",
        "96",
        "196",
        # Smolensk Oblast
        "67",
        # Tambov Oblast
        "68",
        # Tver Oblast
        "69",
        # Tomsk Oblast
        "70",
        # Tula Oblast
        "71",
        # Tyumen Oblast
        "72",
        # Ulyanovsk Oblast
        "73",
        "173",
        # Chelyabinsk Oblast
        "74",
        "174",
        # Zabaykalsky Krai
        "75",
        "80",
        # Yaroslavl Oblast
        "76",
        # Moscow
        "77",
        "97",
        "99",
        "177",
        "197",
        "199",
        "777",
        "799",
        # St. Petersburg
        "78",
        "98",
        "178",
        "198",
        # Jewish Autonomous Oblast
        "79",
        # Agin-Buryat Okrug / "Former Buryat Autonomous District of Aginskoye"
        "80",
        # Komi-Permyak Okrug / "Former Komi-Permyak Autonomous District"
        "81",
        # Republic of Crimea / De jure part of Ukraine as Autonomous Republic. Annexed by Russia in 2014.
        "82",
        # Koryak Okrug / "Former Koryak Autonomous District"
        "82",
        # Nenets Autonomous Okrug (Nenetsia)
        "83",
        # Taymyr Autonomous Okrug / "Former Taymyr (Dolgan-Nenets) Autonomous District"
        "84",
        # Ust-Orda Buryat Okrug / "Former Buryat Autonomous District of Ust-Ordynskoy"
        "85",
        # Khanty-Mansi Autonomous Okrug
        "86",
        "186",
        # Chukotka Autonomous Okrug
        "87",
        # Evenk Autonomous Okrug / "Former Evenk Autonomous District"
        "88",
        # Yamalo-Nenets Autonomous Okrug
        "89",
        # Sevastopol / De jure part of Ukraine as City with special status. Annexed by Russia in 2014.
        "92",
        # Territories outside of the Russian Federation,
        # served by the bodies of internal affairs of the Russian Federation, such as Baikonur
        "94",
    )

    license_plate_formats = (
        # Private vehicle plate
        "{{plate_letter}}{{plate_number}}{{plate_letter}}{{plate_letter}} {{plate_suffix}}",
        # Public transport plate
        "{{plate_letter}}{{plate_letter}}{{plate_number}} {{plate_suffix}}",
        # Trailer plate
        "{{plate_letter}}{{plate_letter}}{{plate_number_extra}} {{plate_suffix}}",
        # Police forces vehicle plate
        "{{plate_letter}}{{plate_number_extra}} {{plate_suffix}}",
        # Military vehicle plate
        "{{plate_number_extra}}{{plate_letter}}{{plate_letter}} {{plate_suffix}}",
        # Diplomatic vehicles
        "{{plate_number_special}} {{plate_suffix}}",
    )

    plate_number_formats = ("###",)

    plate_extra_formats = ("####",)

    plate_special_formats = (
        "00#CD#",
        "00#D###",
        "00#T###",
    )

    def license_plate(self) -> str:
        """Generate a license plate."""
        pattern: str = self.random_element(self.license_plate_formats)
        return self.generator.parse(pattern)

    def plate_letter(self) -> str:
        """Generate a letter for license plates."""
        return self.random_element(self.license_plate_letters)

    def plate_number(self) -> str:
        """Generate a number for license plates."""
        return self.numerify(self.random_element(self.plate_number_formats))

    def plate_number_extra(self) -> str:
        """Generate extra numerical code for license plates."""
        return self.numerify(self.random_element(self.plate_extra_formats))

    def plate_number_special(self) -> str:
        """Generate a special code for license plates."""
        return self.numerify(self.random_element(self.plate_special_formats))

    def plate_suffix(self) -> str:
        """Generate a suffix code for license plates."""
        return self.random_element(self.license_plate_suffix)

    def vehicle_category(self) -> str:
        """Generate a vehicle category code for license plates."""
        return self.random_element(self.vehicle_categories)



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    # Source: https://learnenglish100.com/grammar/career-job/
    jobs = (
        "أحيائي",
        "احصائي",
        "اطفائي",
        "بائع",
        "بائع خضار وفاكهة",
        "بائع زهور",
        "بائعة",
        "بواب",
        "تاجر",
        "جزار",
        "جوھري",
        "جيولوجي",
        "حداد",
        "حلاق",
        "خادمة",
        "خباز",
        "خبير اقتصادي",
        "خبير في التراث الشعبي",
        "خبير في عالم الحيوان",
        "خراط",
        "خياط",
        "خياطة",
        "داية",
        "رئيس طهاه",
        "راقصة",
        "راقصة باليه",
        "رجل مباحث",
        "رسام",
        "روائي",
        "سائق",
        "سائق تاكسي",
        "سائق شاحنة",
        "ساعاتي",
        "ساعي بريد",
        "سكرتير",
        "سكرتيرة",
        "سمكري",
        "سياسي",
        "شاعر",
        "شرطي",
        "صائغ",
        "صاحب متجر",
        "صاحب مطبعة",
        "صاحب مكتبة",
        "صانع أدوات بصرية",
        "صباغ",
        "صباغ أحذية",
        "صحافي",
        "صحفي",
        "صراف",
        "صيدلي",
        "ضابط شرطة",
        "ضارب على الآلة الكاتبة",
        "طباخ",
        "طبيب",
        "طبيب أسنان",
        "طبيب جراح",
        "طبيب عيون",
        "طبيب نفساني",
        "طيار",
        "عارضة أزياء",
        "عالم",
        "عالم أرصاد جوية",
        "عالم اثار",
        "عالم رياضيات",
        "عالم فيزياء",
        "عامل",
        "عامل أحذية",
        "عامل بمتجر",
        "عامل بناء",
        "غسالة",
        "فنان",
        "فيلسوف",
        "قائد شرطة",
        "قاضي",
        "كاتب",
        "كاتب مسرحي",
        "لغوي",
        "مؤلف",
        "ماسح احذية",
        "مبرمج",
        "مترجم",
        "مجلد كتب",
        "محاسب",
        "محاضر",
        "محام",
        "محرر",
        "محرر جريدة",
        "مدير",
        "مدير او مخرج",
        "مدير بنك",
        "مدير تسويق",
        "مدير متجر",
        "مدير موظفين",
        "مذيع",
        "مساعد مبيعات",
        "مشتري",
        "مصحح قانوني",
        "مصصم",
        "مصفف شعر",
        "مصمم جرافيك",
        "مصمم ديكور",
        "مصور",
        "مضيفة جوية",
        "مضيفة في الطائرة",
        "مطرب",
        "معالج طبيعي",
        "معلم",
        "مغني",
        "مكوى",
        "ملحن",
        "ممثل",
        "ممثلة",
        "ممرضة",
        "منتج",
        "منجد",
        "منسق ازياء",
        "موزع جرائد",
        "موسيقار",
        "موصل طلبيات",
        "موظف استقبال",
        "موظف بدالة",
        "موظف حكومي",
        "ميكانيكي",
        "مھندس",
        "نادلة",
        "ناشر",
        "نباتي",
        "نجار",
        "نحات",
        "وسيط تأمين",
        "وكيل سفر",
        "وكيل عقارات",
    )



from itertools import cycle

from .. import Provider as BaseProvider


def rut_check_digit(number: int) -> str:
    """
    Calculate the last character of a RUT number
    :return: RUT check digit
    """

    sum = 0
    for factor in cycle(range(2, 8)):
        if number == 0:
            break
        sum += factor * (number % 10)
        number //= 10
    mod = -sum % 11
    if mod == 11:
        return "0"
    elif mod == 10:
        return "K"
    else:
        return str(mod)


class Provider(BaseProvider):
    """
    A Faker provider for the Chilean VAT IDs, also known as RUTs.

    Sources:

    - https://es.wikipedia.org/wiki/Rol_%C3%9Anico_Tributario - Definition and check digit calculation
    - https://presslatam.cl/2018/04/el-problema-de-la-escasez-y-stock-disponible-de-los-ruts-en-chile/
      paragraph 4, where known ranges are described.
    """

    minimum_rut_person = 10
    maximum_rut_person = 31999999
    minimum_rut_company = 60000000
    maximum_rut_company = 99999999
    rut_format = "{:,d}-{:s}"

    def person_rut(self) -> str:
        """
        :return: a random Chilean RUT between a 10 and 31.999.999 range
        """
        return self.rut(self.minimum_rut_person, self.maximum_rut_person)

    def company_rut(self) -> str:
        """
        :return: a random Chilean RUT between 60.000.000 and 99.999.999
        """
        return self.rut(self.minimum_rut_company, self.maximum_rut_company)

    def rut(self, min: int = minimum_rut_person, max: int = maximum_rut_company) -> str:
        """
        Generates a RUT within the specified ranges, inclusive.

        :param min: Minimum RUT to generate.
        :param max: Maximum RUT to generate.
        :return: a random Chilean RUT between 35.000.000 and 99.999.999
        """

        digits = self.random_int(min, max)
        check = rut_check_digit(digits)
        return self.rut_format.format(digits, check).replace(",", ".")



from .. import Provider as PhoneNumberProvider

# phone number from https://en.wikipedia.org/wiki/Telephone_numbers_in_Taiwan


class Provider(PhoneNumberProvider):
    formats = (
        "(0#) %#######",
        "0#-%#######",
        "0## %#######",
        "09########",
        "09##-######",
        "0#-%######",
        "0# %######",
    )



from .. import Provider as BaseProvider

# Source: ONISEP
# https://www.data.gouv.fr/fr/datasets/r/462f1a98-6f37-4414-9e08-016b78cc4d08


class Provider(BaseProvider):
    jobs = [
        "BIM manager",
        "accessoiriste",
        "accompagnant éducatif et social éducative et sociale",
        "accompagnateur de tourisme équestre",
        "accompagnateur de voyages",
        "accompagnateur en moyenne montagne",
        "acheteur",
        "acheteur d'espaces publicitaires",
        "actuaire",
        "adjoint administratif administrative",
        "administrateur de base de données",
        "administrateur de biens",
        "administrateur de logiciels de laboratoire",
        "administrateur de mission humanitaire",
        "administrateur de spectacle",
        "administrateur judiciaire",
        "administrateur réseaux",
        "administrateur territorial",
        "affûteur",
        "agenceur de cuisines et salles de bains",
        "agent arboricole",
        "agent artistique",
        "agent d'escale",
        "agent d'exploitation de l'eau",
        "agent de constatation des douanes",
        "agent de développement des énergies renouvelables",
        "agent de développement local",
        "agent de développement touristique",
        "agent de propreté et d'hygiène",
        "agent de propreté urbaine",
        "agent de sécurité",
        "agent de sûreté ferroviaire",
        "agent de transit",
        "agent général générale d'assurances",
        "agent hydrothermal hydrothermale",
        "agent immobilier immobilière",
        "agriculteur",
        "agronome",
        "aide-chimiste",
        "aide-soignant",
        "ajusteur-monteur",
        "ambulancier",
        "analyste de crédit",
        "analyste financier",
        "anatomiste",
        "anesthésiste-réanimateur",
        "animalier de laboratoire",
        "animateur 2D et 3D",
        "animateur d'activités physiques et sportives pour tous",
        "animateur de bassin versant",
        "animateur de radio et de télévision",
        "animateur du patrimoine",
        "animateur nature nature",
        "animateur socioculturel socioculturelle",
        "antiquaire",
        "apiculteur",
        "aquaculteur",
        "architecte",
        "architecte d'intérieur",
        "architecte des systèmes d'information",
        "architecte naval",
        "architecte produit industriel",
        "architecte réseaux",
        "architecte web",
        "archiviste",
        "archéologue",
        "art-thérapeute",
        "artiste de cirque",
        "ascensoriste",
        "assistant commercial commerciale",
        "assistant de gestion en PME",
        "assistant de service social",
        "assistant dentaire",
        "assistant en architecture",
        "assistant en ressources humaines",
        "assistant en études de prix",
        "assistant maternel maternelle",
        "assistant réalisateur réalisatrice",
        "astrophysicien",
        "attaché commercial commerciale",
        "attaché d'administration",
        "attaché de presse",
        "attaché de recherche clinique (ARC)",
        "attaché territorial territoriale",
        "audioprothésiste",
        "auditeur externe",
        "auditeur interne",
        "auditeur qualité",
        "auteur-compositeur interprète interprète",
        "auxiliaire de puériculture",
        "auxiliaire spécialisé vétérinaire",
        "avocat",
        "aérodynamicien",
        "bactériologiste",
        "barman",
        "batelier",
        "bibliothécaire",
        "bijoutier-joaillier",
        "bio-informaticien",
        "biologiste en environnement",
        "biologiste médical",
        "biostatisticien",
        "botaniste",
        "bottier",
        "boucher",
        "boulanger",
        "brancardier",
        "brodeur",
        "bronzier",
        "cadreur",
        "caissier",
        "canalisateur",
        "carreleur",
        "carrossier",
        "cartographe",
        "chanteur",
        "charcutier-traiteur",
        "chargé de recherche en recrutement",
        "chargé d'affaires en génie climatique",
        "chargé d'affaires en génie mécanique",
        "chargé d'études en marketing",
        "chargé d'études en valorisation agricole des déchets",
        "chargé d'études média",
        "chargé d'études naturalistes",
        "chargé d'études ressources humaines",
        "chargé d'études économiques",
        "chargé de clientèle banque",
        "chargé de communication interne",
        "chargé de pharmacovigilance",
        "chargé de production",
        "chargé de projet événementiel",
        "chargé de recherche en acoustique musicale",
        "chargé de recherche et développement déchets",
        "chargé de référencement web",
        "chargé de valorisation de la recherche",
        "chargé de veille législative et réglementaire",
        "chargé des méthodes outils et qualité en informatique",
        "chargé des relations publiques",
        "chargé hygiène sécurité environnement (HSE)",
        "charpentier bois",
        "charpentier métallique",
        "chaudronnier",
        "chauffeur de taxi",
        "chef de projet packaging",
        "chef comptable",
        "chef d'exploitation d'usine d'incinération",
        "chef d'exploitation des remontées mécaniques",
        "chef de chantier",
        "chef de chantier en installations électriques",
        "chef de cultures légumières",
        "chef de fabrication des industries graphiques",
        "chef de mission humanitaire",
        "chef de produit marketing",
        "chef de produit technique en informatique",
        "chef de produit touristique",
        "chef de projet biodiversité",
        "chef de projet communication digitale",
        "chef de projet démantèlement nucléaire",
        "chef de projet informatique",
        "chef de projet multimédia",
        "chef de projet sites et sols pollués",
        "chef de projet web/mobile",
        "chef de projet éolien",
        "chef de publicité",
        "chef de rayon",
        "chef de station de traitement des eaux",
        "chef des ventes",
        "chef monteur monteuse",
        "chercheur en biologie",
        "chercheur en biologie du sport",
        "chercheur en chimie",
        "chercheur en physique",
        "chirurgien",
        "chirurgien-dentiste",
        "chocolatier-confiseur",
        "clerc d'huissier",
        "climatologue",
        "coffreur-boiseur",
        "cogniticien",
        "coiffeur",
        "collaborateur de notaire de notaire",
        "collecteur de fonds",
        "coloriste",
        "commercial export",
        "commercial à bord des trains",
        "commerçant en alimentation",
        "commissaire de police",
        "commissaire-priseur",
        "community manager",
        "comptable",
        "comédien",
        "concepteur de jeux vidéo",
        "concepteur de niveaux de jeu web",
        "concepteur designer packaging",
        "concepteur multimédia",
        "concepteur-rédacteur",
        "conducteur d'engins de travaux publics",
        "conducteur d'engins forestiers de récolte en entreprises de travaux forestiers",
        "conducteur de bus ou d'autocar",
        "conducteur de ligne de production alimentaire",
        "conducteur de machine onduleuse",
        "conducteur de machines agricoles",
        "conducteur de machines à imprimer",
        "conducteur de métro",
        "conducteur de train",
        "conducteur de travaux",
        "conducteur de travaux  agricoles",
        "conducteur opérateur opératrice de scierie",
        "conducteur routier routière",
        "conducteur de machines à papier",
        "conseiller agricole",
        "conseiller d'élevage",
        "conseiller en assurances",
        "conseiller en environnement",
        "conseiller en fusions-acquisitions",
        "conseiller en génétique",
        "conseiller en insertion sociale et professionnelle",
        "conseiller en séjour",
        "conseiller en voyages",
        "conseiller en économie sociale et familiale",
        "conseiller espace info-énergie",
        "conseiller principal d'éducation principale d'éducation",
        "conseiller pénitentiaire d'insertion et de probation",
        "conseiller sportif sportive en salle de remise en forme",
        "conservateur du patrimoine",
        "conservateur territorial de bibliothèques",
        "consignataire de navire",
        "constructeur de routes",
        "consultant",
        "consultant SaaS",
        "consultant en conduite de changement",
        "consultant en informatique décisionnelle",
        "consultant en management de l'innovation",
        "consultant en solutions intégrées",
        "consultant en systèmes d'information",
        "consultant en validation",
        "consultant green IT",
        "consultant informatique",
        "contremaître",
        "contrôleur aérien aérienne",
        "contrôleur de gestion",
        "contrôleur de performance",
        "contrôleur des douanes et droits indirects",
        "contrôleur technique automobile",
        "convoyeur de fonds",
        "coordonnateur d'études cliniques",
        "cordiste",
        "cordonnier",
        "correcteur",
        "costumier",
        "courtier",
        "couvreur",
        "credit manager",
        "critique d'art",
        "cryptologue",
        "cuisinier",
        "céramiste",
        "danseur",
        "data manager",
        "designer d'interaction",
        "designer graphique",
        "designer industriel industrielle",
        "designer sonore",
        "dessinateur de BD",
        "dessinateur en construction mécanique",
        "dessinateur-projeteur",
        "diagnostiqueur immobilier",
        "directeur artistique",
        "directeur d'accueil collectif de mineurs (ACM)",
        "directeur d'agence bancaire",
        "directeur d'hôpital",
        "directeur d'hôtel",
        "directeur d'office de tourisme",
        "directeur de création",
        "directeur de golf",
        "directeur de la photographie",
        "directeur de magasin à grande surface",
        "directeur de restaurant",
        "directeur des services pénitentiaires",
        "diététicien",
        "documentaliste",
        "domoticien",
        "déclarant en douane",
        "décolleteur",
        "décorateur",
        "démographe",
        "déménageur",
        "dépanneur en électroménager",
        "développeur d'applications mobiles",
        "développeur informatique",
        "développeur rural rurale humanitaire",
        "développeur économique",
        "employé d'élevage",
        "employé de chai",
        "employé de pressing",
        "employé de restaurant",
        "encadreur",
        "enquêteur privé privée",
        "enseignant d'art",
        "enseignant de la conduite automobile et de la sécurité routière",
        "enseignant humanitaire",
        "enseignant spécialisé spécialisée",
        "enseignant-chercheur",
        "entraîneur de chevaux",
        "entraîneur sportif sportive",
        "ergonome",
        "ergothérapeute",
        "esthéticien-cosméticien",
        "ethnologue",
        "expert bilan carbone",
        "expert automobile",
        "expert en assurances",
        "expert en sécurité informatique",
        "expert immobilier immobilier",
        "expert-comptable",
        "facteur",
        "facteur d'instruments",
        "façadier",
        "façonnier des industries graphiques",
        "femme de chambre",
        "ferronnier d'art",
        "fiscaliste",
        "fleuriste",
        "formateur d'adultes",
        "formateur en informatique",
        "formateur technique en agroéquipement",
        "formulateur",
        "garde (chasse, pêche, littoral, rivière, parcs nationaux)",
        "garde à cheval",
        "gardien de la paix",
        "gardien de police municipale",
        "garçon de café",
        "gendarme",
        "gestionnaire actif/passif",
        "gestionnaire de contrats d'assurance",
        "gestionnaire de contrats informatiques",
        "gestionnaire de données cliniques",
        "gestionnaire de parc micro-informatique",
        "gestionnaire de patrimoine",
        "glaciologue",
        "gouvernant",
        "greffier",
        "grutier",
        "guichetier",
        "guide de haute montagne",
        "guide-conférencier",
        "généalogiste",
        "généticien",
        "géochimiste",
        "géographe",
        "géologue",
        "géologue minier",
        "géologue modélisateur",
        "géomaticien",
        "géomètre-topographe",
        "géophysicien",
        "géotechnicien",
        "géothermicien",
        "gérant de portefeuille",
        "gérant de restauration collective",
        "halieute",
        "histologiste",
        "horloger",
        "horticulteur",
        "hot liner",
        "huissier de justice",
        "hydraulicien",
        "hydrogéologue",
        "hydrologue",
        "hôte d'accueil",
        "hôtesse de l'air",
        "iconographe",
        "illustrateur",
        "infirmier",
        "infirmier humanitaire",
        "informaticien industriel industrielle",
        "ingénieur RD en énergies renouvelables",
        "ingénieur analogicien analogicienne",
        "ingénieur analyste de l'air",
        "ingénieur aromaticien aromaticienne",
        "ingénieur biomédical biomédicale",
        "ingénieur brevets",
        "ingénieur calcul",
        "ingénieur chimiste",
        "ingénieur chimiste en développement analytique",
        "ingénieur cloud computing",
        "ingénieur combustion et brûleurs",
        "ingénieur concepteur conceptrice en mécanique",
        "ingénieur d'affaires en génie électrique",
        "ingénieur d'application",
        "ingénieur d'études en sûreté nucléaire",
        "ingénieur de la police technique et scientifique",
        "ingénieur de maintenance industrielle",
        "ingénieur de recherche (papiers cartons)",
        "ingénieur de recherche clinique et épidémiologique",
        "ingénieur du BTP",
        "ingénieur du son",
        "ingénieur efficacité énergétique du bâtiment",
        "ingénieur en acoustique",
        "ingénieur en automatismes",
        "ingénieur en aéronautique",
        "ingénieur en caractérisation des matériaux",
        "ingénieur en chef territorial",
        "ingénieur en construction automobile",
        "ingénieur en construction navale",
        "ingénieur en fonderie",
        "ingénieur en génie climatique",
        "ingénieur en imagerie médicale",
        "ingénieur en mécanique",
        "ingénieur en métrologie",
        "ingénieur en production et expérimentations végétales",
        "ingénieur en électronique numérique",
        "ingénieur en énergie solaire",
        "ingénieur environnement",
        "ingénieur environnement et risques industriels",
        "ingénieur essais",
        "ingénieur fluides, énergies, réseaux, environnement",
        "ingénieur forage",
        "ingénieur forestier forestière",
        "ingénieur frigoriste",
        "ingénieur gaz",
        "ingénieur hydrogéomorphologue",
        "ingénieur hydroécologue",
        "ingénieur intégration satellite",
        "ingénieur logiciel",
        "ingénieur logistique",
        "ingénieur maintenance aéronautique",
        "ingénieur mathématicien",
        "ingénieur matériaux",
        "ingénieur métallurgiste",
        "ingénieur méthodes mécaniques",
        "ingénieur nucléaire",
        "ingénieur opticien opticienne",
        "ingénieur papetier papetière",
        "ingénieur plasturgiste",
        "ingénieur process aval",
        "ingénieur procédés en chimie",
        "ingénieur production dans les biotechnologies",
        "ingénieur production en aéronautique",
        "ingénieur production en mécanique",
        "ingénieur pétrolier pétrolière",
        "ingénieur qualité moteur",
        "ingénieur radioprotection",
        "ingénieur recherche et développement (R&amp;D) en agroéquipement",
        "ingénieur recherche et développement en agroalimentaire",
        "ingénieur réservoir",
        "ingénieur structures",
        "ingénieur support",
        "ingénieur système",
        "ingénieur systèmes embarqués",
        "ingénieur technico-commercial technico-commerciale",
        "ingénieur technico-commercial technico-commerciale en chimie",
        "ingénieur technico-commercial technico-commerciale en informatique",
        "ingénieur technico-commercial technico-commerciale en électronique",
        "ingénieur textile",
        "ingénieur traitement de l'image",
        "ingénieur télécoms et réseaux",
        "ingénieur écoconcepteur écoconceptrice",
        "ingénieur électricien",
        "ingénieur électronicien électronicienne",
        "ingénieur électronicien électronicienne des systèmes de la sécurité aérienne (IESSA)",
        "ingénieur études et développement en logiciels de simulation",
        "inspecteur de banque",
        "inspecteur des douanes, des finances publiques ou du travail",
        "inspecteur du permis de conduire et de la sécurité routière",
        "installateur en télécoms",
        "inséminateur",
        "intégrateur web",
        "journaliste",
        "journaliste reporter d'images",
        "juge d'instruction",
        "juge des contentieux de la protection",
        "juge des enfants",
        "juriste d'entreprise",
        "juriste droit de l'environnement",
        "juriste en droit social",
        "juriste en propriété intellectuelle",
        "lad-jockey, lad-driver",
        "libraire",
        "linguiste",
        "machiniste-constructeur ou plateau",
        "magasinier cariste",
        "magistrat",
        "manager de risques",
        "mandataire judiciaire",
        "manipulateur en électroradiologie médicale",
        "maquettiste",
        "maquilleur artistique",
        "maraîcher",
        "marchandiseur",
        "maroquinier",
        "maréchal-ferrant",
        "masseur",
        "matelot de la Marine Nationale",
        "matelot de la marine marchande",
        "matelot à la pêche",
        "maçon",
        "maître d'hôtel",
        "maître-chien",
        "menuisier",
        "metteur en scène",
        "microbiologiste",
        "microtechnicien",
        "militaire du rang de l'armée de terre",
        "militaire technicien de l'air (MTA)",
        "miroitier",
        "mixeur son",
        "modiste",
        "modéliste",
        "moniteur d'activités équestres",
        "moniteur de ski",
        "moniteur-éducateur",
        "monteur en installations thermiques et climatiques",
        "monteur en réseaux de distribution électrique",
        "monteur-câbleur",
        "mouleur-noyauteur",
        "musicien",
        "mécanicien bateaux",
        "mécanicien d'entretien d'avion",
        "mécanicien et technicien",
        "mécanicien marine navigant",
        "mécanicien-outilleur",
        "mécanicien-réparateur en matériel agricole",
        "mécatronicien",
        "médecin de secours en montagne",
        "médecin généraliste",
        "médecin humanitaire",
        "médecin spécialiste",
        "médiateur familial familiale",
        "médiateur scientifique",
        "météorologiste",
        "neurobiologiste",
        "nivoculteur",
        "notaire",
        "océanologue",
        "oenologue",
        "officier de gendarmerie",
        "officier de l'armée de l'air",
        "officier de l'armée de terre",
        "officier de la Marine nationale",
        "officier de la marine marchande",
        "officier de police",
        "officier marinier marinière",
        "opticien-lunetier",
        "optronicien",
        "opérateur de fabrication de produits alimentaires",
        "opérateur de raffinerie",
        "opérateur en traitement des matériaux",
        "opérateur prépresse",
        "opérateur sur machine à commande numérique",
        "orfèvre",
        "orthodontiste",
        "orthophoniste",
        "orthoprothésiste",
        "orthoptiste",
        "ostéopathe",
        "ouvrier agricole",
        "ouvrier forestier forestière",
        "ouvrier paysagiste",
        "ouvrier plasturgiste",
        "palefrenier",
        "paléontologue",
        "parfumeur",
        "patron pêcheur",
        "paysagiste",
        "peintre en bâtiment",
        "pharmacien",
        "pharmacien dans l'industrie",
        "photographe",
        "physicien médical médicale",
        "pilote d'hélicoptère",
        "pilote de ligne",
        "pilote de ligne automatisée (chimie - agroalimentaire - industrie pharmaceutique)",
        "pisteur secouriste",
        "plombier",
        "plâtrier",
        "podo-orthésiste",
        "poissonnier",
        "professeur d'éducation physique et sportive (EPS)",
        "professeur dans l'enseignement agricole",
        "professeur de collège et de lycée",
        "professeur de lycée professionnel",
        "professeur de mathématiques ou de physique-chimie",
        "professeur de musique et de danse",
        "professeur des écoles des écoles",
        "professeur en activité physique adaptée",
        "professeur-documentaliste",
        "programmiste",
        "projectionniste",
        "prothésiste dentaire",
        "prototypiste en matériaux souples",
        "préparateur en pharmacie",
        "psychanalyste",
        "psychologue",
        "psychologue de l'Éducation nationale spécialité éducation, développement et apprentissages",
        "psychologue de l'éducation nationale spécialité éducation, développement et conseil en"
        + " orientation scolaire et professionnelle",
        "psychomotricien",
        "puériculteur",
        "pâtissier",
        "pédiatre",
        "pédicure-podologue",
        "pédologue",
        "relieur-doreur",
        "reporter-photographe",
        "responsable achats en chimie",
        "responsable approvisionnement",
        "responsable assurance qualité",
        "responsable biométrie",
        "responsable d'élevage agricole",
        "responsable de fabrication en chimie",
        "responsable de formation",
        "responsable de la collecte des déchets ménagers",
        "responsable de la promotion des ventes",
        "responsable de laboratoire de contrôle en biologie",
        "responsable de laboratoire de contrôle en chimie",
        "responsable de laboratoire de recherche",
        "responsable de plate-forme biotechnologique",
        "responsable de production alimentaire",
        "responsable de projets culturels",
        "responsable de rémunération",
        "responsable de réseau d'assainissement",
        "responsable de réseau eau potable",
        "responsable de scierie",
        "responsable de site de traitement des déchets",
        "responsable des ouvrages hydroélectriques",
        "responsable des produits structurés actions",
        "responsable des ressources humaines",
        "responsable du back office",
        "responsable du recrutement",
        "responsable du service après-vente",
        "responsable du soutien logistique intégré",
        "responsable e-CRM",
        "responsable qualité en agroalimentaire",
        "restaurateur d'oeuvres d'art",
        "roboticien",
        "rudologue",
        "réceptionniste",
        "rédacteur en chef",
        "rédacteur médical médicale",
        "rédacteur on line",
        "rédacteur territorial territoriale",
        "réflexologue",
        "régisseur de spectacles",
        "régisseur général générale cinéma",
        "régisseur lumière",
        "régleur",
        "sage-femme",
        "salesman",
        "sapeur-pompier",
        "scripte",
        "sculpteur sur bois",
        "scénariste",
        "secrétaire",
        "secrétaire administratif",
        "secrétaire d'édition",
        "secrétaire de rédaction",
        "secrétaire des affaires étrangères",
        "secrétaire juridique",
        "secrétaire médical",
        "sellier",
        "serrurier dépanneur dépanneuse",
        "serrurier-métallier",
        "sociologue",
        "soigneur d'animaux",
        "solier-moquettiste",
        "sommelier",
        "soudeur",
        "souffleur de verre",
        "sous-officier de l'armée de l'air",
        "sous-officier de l'armée de terre",
        "souscripteur",
        "sportif de haut niveau",
        "spécialiste de l'accessibilité numérique",
        "spécialiste des affaires réglementaires en chimie",
        "staffeur-ornemaniste",
        "statisticien",
        "statisticien en analyse sensorielle",
        "statisticien en géomarketing",
        "statisticien industriel industrielle",
        "styliste",
        "substitut du procureur",
        "surveillant de centre pénitentiaire",
        "syndic de copropriété",
        "sérigraphe",
        "tailleur de pierre",
        "tailleur-couturier",
        "tapissier d'ameublement",
        "technicien automobile",
        "technicien biologiste",
        "technicien chimiste",
        "technicien céramiste",
        "technicien d'analyses biomédicales",
        "technicien d'essais",
        "technicien d'exploitation de l'eau",
        "technicien d'exploitation du réseau gaz",
        "technicien d'intervention clientèle gaz",
        "technicien de contrôle",
        "technicien de fabrication de mobilier et de menuiserie",
        "technicien de forge",
        "technicien de l'intervention sociale et familiale",
        "technicien de la circulation ferroviaire SNCF",
        "technicien de maintenance en génie climatique",
        "technicien de maintenance en informatique",
        "technicien de maintenance industrielle",
        "technicien de police technique et scientifique",
        "technicien des industries du verre",
        "technicien démonstrateur démonstratrice en matériel agricole",
        "technicien en automatismes",
        "technicien en engins de travaux publics",
        "technicien en lignes haute tension",
        "technicien en métrologie",
        "technicien en optique de précision",
        "technicien en traitement des déchets",
        "technicien en traitement des matériaux",
        "technicien forestier forestière",
        "technicien logistique",
        "technicien packaging",
        "technicien paysagiste",
        "technicien plasturgiste",
        "technicien prototypiste en agroéquipement",
        "technicien pétrolier pétrolière",
        "technicien qualité",
        "technicien radioprotection",
        "technicien réalisateur réalisatrice radio",
        "technicien thermicien thermicienne",
        "technicien télécoms et réseaux",
        "technicien électronicien électronicienne",
        "technicien électrotechnicien électrotechnicienne",
        "technico-commercial en agroalimentaire",
        "teinturier blanchisseur",
        "testeur",
        "toiletteur d'animaux",
        "tonnelier",
        "trader",
        "traducteur technique",
        "traducteur-interprète",
        "traffic manager",
        "télévendeur",
        "urbaniste",
        "veilleur stratégique",
        "vendeur conseil caviste",
        "vendeur en animalerie",
        "vendeur en magasin",
        "vendeur en micro-informatique et multimédia",
        "vendeur-conseil en matériel agricole",
        "vendeur-magasinier en fournitures automobiles",
        "verrier au chalumeau",
        "visiteur médical médicale",
        "viticulteur",
        "vitrailliste",
        "volcanologue",
        "vétérinaire",
        "web-ergonome",
        "webdesigner",
        "webmestre",
        "yield manager",
        "zoologiste",
        "ébéniste",
        "éclairagiste",
        "économe de flux",
        "économiste de la construction",
        "économètre statisticien",
        "écrivain",
        "éditeur",
        "éducateur canin canine",
        "éducateur de jeunes enfants",
        "éducateur de la protection judiciaire de la jeunesse (PJJ)",
        "éducateur sportif sportive",
        "éducateur sportif sportive des activités aquatiques et de la natation",
        "éducateur spécialisé spécialisée",
        "éducateur technique spécialisé technique spécialisée",
        "élagueur",
        "électricien installateur installatrice",
        "électromécanicien",
        "électromécanicien en remontées mécaniques",
        "électronicien automobile",
        "énergéticien",
        "étalagiste",
        "étanchéiste",
    ]



from typing import Tuple

from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_formats = ("{{city_name}}",)
    building_number_formats = ("%", "%#", "%#", "%#", "%##")
    street_suffixes = ["strasse"]
    street_name_formats = ("{{last_name}}{{street_suffix}}",)
    street_address_formats = ("{{street_name}} {{building_number}}",)
    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)
    postcode_formats = (
        "1###",
        "2###",
        "3###",
        "4###",
        "5###",
        "6###",
        "7###",
        "8###",
        "9###",
    )

    cities = (
        "Aarau",
        "Adliswil",
        "Aesch",
        "Affoltern",
        "Aigle",
        "Allschwil",
        "Altstätten",
        "Amriswil",
        "Arbon",
        "Arth",
        "Baar",
        "Baden",
        "Basel",
        "Bassersdorf",
        "Bellinzona",
        "Belp",
        "Bern",
        "Bernex",
        "Biel/Bienne",
        "Binningen",
        "Birsfelden",
        "Brig-Glis",
        "Brugg",
        "Buchs",
        "Bülach",
        "Bulle",
        "Burgdorf",
        "Carouge",
        "Cham",
        "Chêne-Bougeries",
        "Chur",
        "Crans-Montana",
        "Davos",
        "Delsberg",
        "Dietikon",
        "Dübendorf",
        "Ebikon",
        "Ecublens",
        "Einsiedeln",
        "Emmen",
        "Flawil",
        "Frauenfeld",
        "Freiburg",
        "Freienbach",
        "Genf",
        "Gland",
        "Glarus",
        "Glarus",
        "Gossau",
        "Gossau",
        "Grenchen",
        "Herisau",
        "Hinwil",
        "Horgen",
        "Horw",
        "Illnau-Effretikon",
        "Ittigen",
        "Kloten",
        "Köniz",
        "Kreuzlingen",
        "Kriens",
        "Küsnacht",
        "Küssnacht",
        "La Chaux-de-Fonds",
        "La Tour-de-Peilz",
        "Lancy",
        "Langenthal",
        "Lausanne",
        "Le Grand-Saconnex",
        "Lenzburg",
        "Liestal",
        "Locarno",
        "Lugano",
        "Lutry",
        "Luzern",
        "Lyss",
        "Männedorf",
        "Martigny",
        "Maur",
        "Meilen",
        "Mendrisio",
        "Meyrin",
        "Möhlin",
        "Monthey",
        "Montreux",
        "Morges",
        "Münchenbuchsee",
        "Münchenstein",
        "Münsingen",
        "Muri",
        "Muttenz",
        "Naters",
        "Neuenburg",
        "Neuhausen",
        "Nyon",
        "Oberwil",
        "Oftringen",
        "Olten",
        "Onex",
        "Opfikon",
        "Ostermundigen",
        "Payerne",
        "Pfäffikon",
        "Plan-les-Ouates",
        "Pratteln",
        "Prilly",
        "Pully",
        "Rapperswil-Jona",
        "Regensdorf",
        "Reinach",
        "Renens",
        "Rheinfelden",
        "Richterswil",
        "Riehen",
        "Risch",
        "Romanshorn",
        "Rüti",
        "Sarnen",
        "Schaffhausen",
        "Schlieren",
        "Schwyz",
        "Siders",
        "Sitten",
        "Solothurn",
        "Spiez",
        "Spreitenbach",
        "St. Gallen",
        "Stäfa",
        "Steffisburg",
        "Steinhausen",
        "Suhr",
        "Sursee",
        "Thalwil",
        "Thônex",
        "Thun",
        "Urdorf",
        "Uster",
        "Uzwil",
        "Val-de-Ruz",
        "Val-de-Travers",
        "Vernier",
        "Versoix",
        "Vevey",
        "Veyrier",
        "Villars-sur-Glâne",
        "Volketswil",
        "Wädenswil",
        "Wald",
        "Wallisellen",
        "Weinfelden",
        "Wettingen",
        "Wetzikon",
        "Wil",
        "Winterthur",
        "Wohlen",
        "Worb",
        "Yverdon-les-Bains",
        "Zofingen",
        "Zollikofen",
        "Zollikon",
        "Zug",
        "Zürich",
    )

    cantons = (
        ("AG", "Aargau"),
        ("AI", "Appenzell Innerrhoden"),
        ("AR", "Appenzell Ausserrhoden"),
        ("BE", "Bern"),
        ("BL", "Basel-Landschaft"),
        ("BS", "Basel-Stadt"),
        ("FR", "Freiburg"),
        ("GE", "Genf"),
        ("GL", "Glarus"),
        ("GR", "Graubünden"),
        ("JU", "Jura"),
        ("LU", "Luzern"),
        ("NE", "Neuenburg"),
        ("NW", "Nidwalden"),
        ("OW", "Obwalden"),
        ("SG", "St. Gallen"),
        ("SH", "Schaffhausen"),
        ("SO", "Solothurn"),
        ("SZ", "Schwyz"),
        ("TG", "Thurgau"),
        ("TI", "Tessin"),
        ("UR", "Uri"),
        ("VD", "Waadt"),
        ("VS", "Wallis"),
        ("ZG", "Zug"),
        ("ZH", "Zürich"),
    )

    def canton(self) -> Tuple[str, str]:
        """
        Randomly returns a swiss canton ('Abbreviated', 'Name').
        :example ('ZH', 'Zürich')
        """
        return self.random_element(self.cantons)

    def city_name(self) -> str:
        """
        Randomly returns a swiss city.
        :example 'Zug'
        """
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        """
        Randomly returns a Swiss canton name.
        :example 'Zürich'
        """
        return self.canton()[1]

    canton_name = administrative_unit

    def canton_code(self) -> str:
        """
        Randomly returns a Swiss canton code.
        :example 'ZH'
        """
        return self.canton()[0]



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``en_CA`` locale.

    Sources:

    - https://www.revolvy.com/main/index.php?s=Canadian%20licence%20plate%20designs%20and%20serial%20formats
    """

    license_formats = (
        # Alberta
        "???-####",
        # BC
        "??# ##?",
        "?? ####",
        # Manitoba
        "??? ###",
        # New Brunswick
        "??? ###",
        # Newfoundland and Labrador
        "??? ###",
        # NWT
        "######",
        # Nova Scotia
        "??? ###",
        # Nunavut
        "### ###",
        # Ontario
        "### ???",
        "???? ###",
        "??# ###",
        "### #??",
        "?? ####",
        "GV??-###",
        # PEI
        "## ##??",
        # Quebec
        "?## ???",
        # Saskatchewan
        "### ???",
        # Yukon
        "???##",
    )



from typing import Any

from ..en_CA import Provider as EnCaProvider


class Provider(EnCaProvider):
    #  Most of the parts are identical to en_CA, we simply override those who are not shared between the two.

    city_prefixes = (
        "Ville",
        "Baie",
        "Saint-",
        "Sainte-",
        "Mont-",
        "La",
        "Lac-",
        "L'",
        "L'Île-",
    )

    city_suffixes = (
        "Est",
        "Ouest",
        "-sur-Mer",
    )

    street_prefixes = (
        "rue",
        "rue",
        "chemin",
        "avenue",
        "boulevard",
        "route",
        "rang",
        "allé",
        "montée",
    )

    provinces = (
        "Alberta",
        "Colombie-Britannique",
        "Manitoba",
        "Nouveau-Brunswick",
        "Terre-Neuve-et-Labrador",
        "Territoires du Nord-Ouest",
        "Nouvelle-Écosse",
        "Nunavut",
        "Ontario",
        "Île-du-Prince-Édouard",
        "Québec",
        "Saskatchewan",
        "Yukon",
    )

    street_name_formats = (
        "{{street_prefix}} {{first_name}}",
        "{{street_prefix}} {{last_name}}",
    )

    city_formats = (
        "{{city_prefix}} {{last_name}}",
        "{{city_prefix}} {{last_name}}",
        "{{city_prefix}}-{{city_prefix}}-{{last_name}}",
        "{{city_prefix}} {{first_name}} {{city_suffix}}",
        "{{city_prefix}} {{first_name}}",
        "{{city_prefix}} {{first_name}}",
        "{{city_prefix}} {{first_name}}",
        "{{last_name}}",
        "{{last_name}}",
        "{{first_name}} {{city_suffix}}",
        "{{last_name}} {{city_suffix}}",
    )

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

    def street_prefix(self) -> str:
        """
        :example: 'rue'
        """
        return self.random_element(self.street_prefixes)



from .. import Provider as BankProvider


class Provider(BankProvider):
    """Implement bank provider for ``es_AR`` locale.
    source: https://www.bcra.gob.ar/SistemasFinancierosYdePagos/Activos.asp"""

    bban_format = "????####################"
    country_code = "AR"

    banks = (
        "Banco de la Nación Argentina",
        "Banco Santander",
        "Banco de Galicia y Buenos Aires",
        "Banco de la Provincia de Buenos Aires",
        "BBVA Argentina",
        "Banco Macro",
        "HSBC Bank Argentina",
        "Banco Ciudad de Buenos Aires",
        "Banco Credicoop",
        "Industrial And Commercial Bank Of China",
        "Citibank",
        "Banco Patagonia",
        "Banco de la Provincia de Córdoba",
        "Banco Supervielle",
        "Nuevo Banco de Santa Fe",
        "Banco Hipotecario S. A.",
        "Banco Itaú Argentina",
        "Banco de Inversión y Comercio Exterior (BICE)",
        "Banco Comafi",
        "BSE - Banco Santiago del Estero",
    )

    def bank(self) -> str:
        """Generate a bank name."""
        return self.random_element(self.banks)



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{last_name}}, {{first_name}}",
    )

    first_names = (
        "Ādams",
        "Ādolfs",
        "Agris",
        "Aigars",
        "Ainārs",
        "Aivars",
        "Alberts",
        "Aldis",
        "Aleksandrs",
        "Alfrēds",
        "Andrejs",
        "Andris",
        "Andrešs",
        "Ansis",
        "Antons",
        "Armands",
        "Arnis",
        "Arnolds",
        "Artis",
        "Arturs",
        "Artūrs",
        "Arvīds",
        "Augusts",
        "Bērends",
        "Bērtulis",
        "Brencis",
        "Dainis",
        "Daniels",
        "Dāvis",
        "Dzintars",
        "Edgars",
        "Edmunds",
        "Eduards",
        "Edvīns",
        "Egils",
        "Elmārs",
        "Elvis",
        "Emīls",
        "Ēriks",
        "Ermanis",
        "Ernests",
        "Ēvalds",
        "Fricis",
        "Gatis",
        "Gunārs",
        "Guntars",
        "Guntis",
        "Ģederts",
        "Ģirts",
        "Hanss",
        "Harijs",
        "Henriks",
        "Hermanis",
        "Igors",
        "Ilmārs",
        "Imants",
        "Indriķis",
        "Ivars",
        "Ivo",
        "Jakobs",
        "Janis",
        "Jānis",
        "Jannis",
        "Jāzeps",
        "Jēkabs",
        "Jēkaubs",
        "Jezups",
        "Johans",
        "Jūlijs",
        "Juris",
        "Kārlis",
        "Kaspars",
        "Konradus",
        "Kristaps",
        "Kristers",
        "Krists",
        "Krišjānis",
        "Krišs",
        "Laimonis",
        "Lauris",
        "Leons",
        "Macs",
        "Mareks",
        "Māris",
        "Mārtiņš",
        "Matīss",
        "Mihels",
        "Mikels",
        "Miķelis",
        "Modris",
        "Nikolajs",
        "Niks",
        "Normunds",
        "Oļģerts",
        "Oskars",
        "Osvalds",
        "Oto",
        "Pauls",
        "Pēteris",
        "Raimonds",
        "Raivis",
        "Reinis",
        "Ričards",
        "Rihards",
        "Roberts",
        "Rolands",
        "Rūdolfs",
        "Sandis",
        "Staņislavs",
        "Tenis",
        "Teodors",
        "Toms",
        "Uldis",
        "Valdis",
        "Viesturs",
        "Viktors",
        "Vilis",
        "Vilnis",
        "Viļums",
        "Visvaldis",
        "Vladislavs",
        "Voldemārs",
        "Ziedonis",
        "Žanis",
        "Agnese",
        "Aiga",
        "Aija",
        "Aina",
        "Alīda",
        "Alise",
        "Alma",
        "Alvīne",
        "Amālija",
        "Anete",
        "Anita",
        "Anna",
        "Annija",
        "Antoņina",
        "Antra",
        "Ārija",
        "Ausma",
        "Austra",
        "Baba",
        "Baiba",
        "Berta",
        "Biruta",
        "Broņislava",
        "Dace",
        "Daiga",
        "Daina",
        "Dārta",
        "Diāna",
        "Doroteja",
        "Dzidra",
        "Dzintra",
        "Eda",
        "Edīte",
        "Elīna",
        "Elita",
        "Elizabete",
        "Elvīra",
        "Elza",
        "Emīlija",
        "Emma",
        "Ērika",
        "Erna",
        "Eva",
        "Evija",
        "Evita",
        "Gaida",
        "Genovefa",
        "Grēta",
        "Grieta",
        "Gunita",
        "Gunta",
        "Helēna",
        "Ieva",
        "Ilga",
        "Ilona",
        "Ilze",
        "Ina",
        "Ināra",
        "Indra",
        "Inese",
        "Ineta",
        "Inga",
        "Ingrīda",
        "Inguna",
        "Inta",
        "Irēna",
        "Irma",
        "Iveta",
        "Jana",
        "Janina",
        "Jūle",
        "Jūla",
        "Jūlija",
        "Karina",
        "Karlīna",
        "Katarīna",
        "Katrīna",
        "Krista",
        "Kristiāna",
        "Laila",
        "Laura",
        "Lavīze",
        "Leontīne",
        "Lība",
        "Lidija",
        "Liene",
        "Līga",
        "Ligita",
        "Lilija",
        "Lilita",
        "Līna",
        "Linda",
        "Līza",
        "Lizete",
        "Lūcija",
        "Madara",
        "Made",
        "Maija",
        "Māra",
        "Mare",
        "Margareta",
        "Margrieta",
        "Marija",
        "Mārīte",
        "Marta",
        "Maža",
        "Milda",
        "Minna",
        "Mirdza",
        "Monika",
        "Natālija",
        "Olga",
        "Otīlija",
        "Paula",
        "Paulīna",
        "Rasma",
        "Regīna",
        "Rita",
        "Rudīte",
        "Ruta",
        "Rute",
        "Samanta",
        "Sandra",
        "Sanita",
        "Santa",
        "Sapa",
        "Sarmīte",
        "Silvija",
        "Sintija",
        "Skaidrīte",
        "Solvita",
        "Tekla",
        "Trīne",
        "Valda",
        "Valentīna",
        "Valija",
        "Velta",
        "Veneranda",
        "Vera",
        "Veronika",
        "Vija",
        "Vilma",
        "Vineta",
        "Vita",
        "Zane",
        "Zelma",
        "Zenta",
        "Zigrīda",
    )

    last_names = (
        "Ābele",
        "Āboliņš",
        "Ābols",
        "Alksnis",
        "Apinis",
        "Apsītis",
        "Auniņš",
        "Auziņš",
        "Avotiņš",
        "Balodis",
        "Baltiņš",
        "Bērziņš",
        "Birznieks",
        "Bite",
        "Briedis",
        "Caune",
        "Celmiņš",
        "Celms",
        "Cīrulis",
        "Dzenis",
        "Dūmiņš",
        "Eglītis",
        "Jaunzems",
        "Kalējs",
        "Kalniņš",
        "Kaņeps",
        "Kārkliņš",
        "Kauliņš",
        "Kļaviņš",
        "Krastiņš",
        "Krēsliņš",
        "Krieviņš",
        "Krievs",
        "Krūmiņš",
        "Krūze",
        "Kundziņš",
        "Lācis",
        "Lagzdiņš",
        "Lapsa",
        "Līcis",
        "Liepa",
        "Liepiņš",
        "Lukstiņš",
        "Lūsis",
        "Paegle",
        "Pērkons",
        "Podnieks",
        "Polis",
        "Priede",
        "Priedītis",
        "Puriņš",
        "Purmals",
        "Riekstiņš",
        "Roze",
        "Rozītis",
        "Rubenis",
        "Rudzītis",
        "Saulītis",
        "Siliņš",
        "Skuja",
        "Skujiņš",
        "Sproģis",
        "Strazdiņš",
        "Turiņš",
        "Vanags",
        "Vīksna",
        "Vilciņš",
        "Vilks",
        "Vītoliņš",
        "Vītols",
        "Zaķis",
        "Zālītis",
        "Zariņš",
        "Zeltiņš",
        "Ziemelis",
        "Zirnis",
        "Zvaigzne",
        "Zvirbulis",
    )



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``sv_SE`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Sweden
    - https://www.transportstyrelsen.se/en/road/Vehicles/license-plates/
    """

    license_formats = (
        # Classic format
        "??? ###",
        # New format
        "??? ##?",
    )



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """Implement color provider for ``es`` locale."""

    all_colors = OrderedDict(
        (
            ("Agua marina medio", "#66CDAA"),
            ("Agua-marina", "#7FFFD4"),
            ("Almendra blanqueado", "#FFEBCD"),
            ("Amarillo", "#FFFF00"),
            ("Amarillo claro", "#FFFFE0"),
            ("Amarillo dorado", "#DAA520"),
            ("Amarillo dorado claro", "#FAFAD2"),
            ("Amarillo dorado oscuro", "#B8860B"),
            ("Amarillo dorado pálido", "#EEE8AA"),
            ("Amarillo trigo", "#F5DEB3"),
            ("Amarillo verde", "#9ACD32"),
            ("Azul", "#0000FF"),
            ("Azul Alicia", "#F0F8FF"),
            ("Azul acero", "#4682B4"),
            ("Azul acero claro", "#B0C4DE"),
            ("Azul anciano", "#6495ED"),
            ("Azul azur", "#F0FFFF"),
            ("Azul cadete", "#5F9EA0"),
            ("Azul cielo", "#87CEEB"),
            ("Azul cielo claro", "#87CEFA"),
            ("Azul cielo profundo", "#00BFFF"),
            ("Azul claro", "#ADD8E6"),
            ("Azul lona", "#1E90FF"),
            ("Azul marino", "#000080"),
            ("Azul medianoche", "#191970"),
            ("Azul medio", "#0000CD"),
            ("Azul oscuro", "#00008B"),
            ("Azul pizarra", "#6A5ACD"),
            ("Azul pizarra medio", "#7B68EE"),
            ("Azul pizarra oscuro", "#483D8B"),
            ("Azul polvo", "#B0E0E6"),
            ("Azul real", "#4169E1"),
            ("Azul violeta", "#8A2BE2"),
            ("Beige", "#F5F5DC"),
            ("Beige antiguo", "#FAEBD7"),
            ("Beige limón", "#FFFACD"),
            ("Beige melocotón", "#FFDAB9"),
            ("Beige mocasín", "#FFE4B5"),
            ("Beige papaya", "#FFEFD5"),
            ("Bisque", "#FFE4C4"),
            ("Blanco", "#FFFFFF"),
            ("Blanco concha", "#FFF5EE"),
            ("Blanco encaje", "#FDF5E6"),
            ("Blanco fantasma", "#F8F8FF"),
            ("Blanco floral", "#FFFAF0"),
            ("Blanco humo", "#F5F5F5"),
            ("Blanco lavanda", "#FFF0F5"),
            ("Blanco lino", "#FAF0E6"),
            ("Blanco menta", "#F5FFFA"),
            ("Blanco navajo", "#FFDEAD"),
            ("Blanco nieve", "#FFFAFA"),
            ("Caqui", "#6B8E23"),
            ("Caqui oscuro", "#BDB76B"),
            ("Chartreuse", "#7FFF00"),
            ("Chocolate", "#D2691E"),
            ("Cian", "#00FFFF"),
            ("Cian clarto", "#E0FFFF"),
            ("Ciruela", "#DDA0DD"),
            ("Coral", "#FF7F50"),
            ("Coral claro", "#F08080"),
            ("Amarillo maíz dulce", "#FFF8DC"),
            ("Cyan oscuro", "#008B8B"),
            ("Fucsia", "#FF00FF"),
            ("Granate", "#800000"),
            ("Gris", "#808080"),
            ("Gris claro", "#D3D3D3"),
            ("Gris gainsboro (Estaño)", "#DCDCDC"),
            ("Gris mate", "#696969"),
            ("Gris oscuro", "#A9A9A9"),
            ("Gris pizarra", "#708090"),
            ("Gris pizarra claro", "#778899"),
            ("Gris pizarra oscuro", "#2F4F4F"),
            ("Lavanda", "#E6E6FA"),
            ("Lima", "#00FF00"),
            ("Magenta", "#FF00FF"),
            ("Magenta oscuro", "#8B008B"),
            ("Marfil", "#FFFFF0"),
            ("Marrón", "#A52A2A"),
            ("Marrón arena", "#F4A460"),
            ("Marrón caqui", "#F0E68C"),
            ("Marrón cuero", "#8B4513"),
            ("Marrón madera rústica", "#DEB887"),
            ("Marrón perú", "#CD853F"),
            ("Marrón rojizo", "#D2B48C"),
            ("Marrón rosado", "#BC8F8F"),
            ("Marrón siena", "#A0522D"),
            ("Melón dulce", "#F0FFF0"),
            ("Naranja", "#FFA500"),
            ("Naranja oscuro", "#FF8C00"),
            ("Negro", "#000000"),
            ("Oliva", "#808000"),
            ("Oro", "#FFD700"),
            ("Orquídea", "#DA70D6"),
            ("Orquídea medio", "#BA55D3"),
            ("Orquídea púrpura oscuro", "#9932CC"),
            ("Plata", "#C0C0C0"),
            ("Púrpura", "#800080"),
            ("Púrpura medio", "#9370DB"),
            ("Rojo", "#FF0000"),
            ("Rojo anaranjado", "#FF4500"),
            ("Rojo carmesí", "#DC143C"),
            ("Rojo indio", "#CD5C5C"),
            ("Rojo ladrillo", "#B22222"),
            ("Rojo oscuro", "#8B0000"),
            ("Rojo tomate", "#FF6347"),
            ("Rojo violeta medio", "#C71585"),
            ("Rosa", "#FFC0CB"),
            ("Rosa brumoso", "#FFE4E1"),
            ("Rosa caliente", "#FF69B4"),
            ("Rosa claro", "#FFB6C1"),
            ("Rosa profundo", "#FF1493"),
            ("Salmón", "#FA8072"),
            ("Salmón claro", "#FFA07A"),
            ("Salmón oscuro", "#E9967A"),
            ("Turquesa", "#40E0D0"),
            ("Turquesa medio", "#48D1CC"),
            ("Turquesa oscuro", "#00CED1"),
            ("Turquesa pálido", "#AFEEEE"),
            ("Verde", "#008000"),
            ("Verde azulado", "#008080"),
            ("Verde bosque", "#228B22"),
            ("Verde claro", "#90EE90"),
            ("Verde lima", "#32CD32"),
            ("Verde limón", "#ADFF2F"),
            ("Verde mar", "#2E8B57"),
            ("Verde mar claro", "#20B2AA"),
            ("Verde mar medio", "#3CB371"),
            ("Verde mar oscuro", "#8FBC8F"),
            ("Verde oliva oscuro", "#556B2F"),
            ("Verde oscuro", "#006400"),
            ("Verde prado", "#7CFC00"),
            ("Verde primavera", "#00FF7F"),
            ("Verde primavera medio", "#00FA9A"),
            ("Verde pálido", "#98FB98"),
            ("Violeta", "#EE82EE"),
            ("Violeta cardo", "#D8BFD8"),
            ("Violeta oscuro", "#9400D3"),
            ("Violeta sonrojado pálido", "#DB7093"),
            ("Índigo", "#4B0082"),
        )
    )

    safe_colors = (
        "negro",
        "budeos",
        "verde",
        "rojo",
        "violeta",
        "verde azulado",
        "azul",
        "plata",
        "gris",
        "amarilo",
        "fucsia",
        "cian",
        "blanco",
    )



import re

from itertools import product
from typing import Dict, Optional, Pattern

from .. import PrefixType
from .. import Provider as BarcodeProvider


class Provider(BarcodeProvider):
    """Implement barcode provider for ``en_US`` locale.

    Sources:

    - https://gs1.org/standards/id-keys/company-prefix
    """

    local_prefixes = (
        *product((0,), range(10)),
        *product((1,), range(4)),
    )

    upc_e_base_pattern: Pattern = re.compile(r"^\d{6}$")
    upc_ae_pattern1: Pattern = re.compile(
        r"^(?P<number_system_digit>[01])"  # The first digit must be 0 or 1
        r"(?=\d{11}$)"  # followed by 11 digits of which
        r"(?P<mfr_code>\d{2})"  # the first 2 digits make up the manufacturer code,
        r"(?:(?P<extra>[012])0{4})"  # if immediately followed by 00000, 10000, or 20000,
        r"(?P<product_code>\d{3})"  # a 3-digit product code,
        r"(?P<check_digit>\d)$",  # and finally a check digit.
    )
    upc_ae_pattern2: Pattern = re.compile(
        r"^(?P<number_system_digit>[01])"  # The first digit must be 0 or 1
        r"(?=\d{11}$)"  # followed by 11 digits of which
        r"(?P<mfr_code>\d{3,4}?)"  # the first 3 or 4 digits make up the manufacturer code,
        r"(?:0{5})"  # if immediately followed by 00000,
        r"(?P<product_code>\d{1,2})"  # a 2-digit or single digit product code,
        r"(?P<check_digit>\d)$",  # and finally a check digit.
    )
    upc_ae_pattern3: Pattern = re.compile(
        r"^(?P<number_system_digit>[01])"  # The first digit must be 0 or 1
        r"(?=\d{11}$)"  # followed by 11 digits of which
        r"(?P<mfr_code>\d{5})"  # the first 5 digits make up the manufacturer code,
        r"(?:0{4}(?P<extra>[5-9]))"  # if immediately followed by 0000 and a 5, 6, 7, 8, or 9,
        r"(?P<check_digit>\d)$",  # and finally a check digit.
    )

    def ean13(self, prefixes: PrefixType = (), leading_zero: Optional[bool] = None) -> str:
        """Generate an EAN-13 barcode.

        If ``leading_zero`` is ``True``, the leftmost digit of the barcode will
        be set to ``0``. If ``False``, the leftmost digit cannot be ``0``. If
        ``None`` (default), the leftmost digit can be any digit.

        If a value for ``prefixes`` is specified, the result will begin with one
        of the sequences in ``prefixes`` and will ignore ``leading_zero``.

        This method uses the standard barcode provider's |ean13| under the
        hood with the ``prefixes`` argument set to the correct value to attain
        the behavior described above.

        .. note::
           EAN-13 barcode that starts with a zero can be converted to UPC-A
           by dropping the leading zero. This may cause problems with readers
           that treat all of these code as UPC-A codes and drop the first digit
           when reading it.

           You can set the argument ``prefixes`` ( or ``leading_zero`` for
           convenience) explicitly to avoid or to force the generated barcode to
           start with a zero. You can also generate actual UPC-A barcode with
           |EnUsBarcodeProvider.upc_a|.

        :sample:
        :sample: leading_zero=False
        :sample: leading_zero=True
        :sample: prefixes=('00',)
        :sample: prefixes=('45', '49')
        """
        if not prefixes:
            if leading_zero is True:
                prefixes = ((0,),)
            elif leading_zero is False:
                prefixes = ((self.random_int(1, 9),),)

        return super().ean13(prefixes=prefixes)

    def _convert_upc_a2e(self, upc_a: str) -> str:
        """Convert a 12-digit UPC-A barcode to its 8-digit UPC-E equivalent.

        .. warning::
           Not all UPC-A barcodes can be converted.
        """
        if not isinstance(upc_a, str):
            raise TypeError("`upc_a` is not a string")
        m1 = self.upc_ae_pattern1.match(upc_a)
        m2 = self.upc_ae_pattern2.match(upc_a)
        m3 = self.upc_ae_pattern3.match(upc_a)
        if not any([m1, m2, m3]):
            raise ValueError("`upc_a` has an invalid value")
        upc_e_template = "{number_system_digit}{mfr_code}{product_code}{extra}{check_digit}"
        if m1:
            upc_e = upc_e_template.format(**m1.groupdict())
        elif m2:
            groupdict: Dict[str, str] = m2.groupdict()
            mfr_code = groupdict.get("mfr_code") or ""
            groupdict["extra"] = str(len(mfr_code))
            upc_e = upc_e_template.format(**groupdict)
        elif m3:
            groupdict = m3.groupdict()
            groupdict["product_code"] = ""
            upc_e = upc_e_template.format(**groupdict)
        return upc_e

    def _upc_ae(self, base: Optional[str] = None, number_system_digit: Optional[int] = None) -> str:
        """Create a 12-digit UPC-A barcode that can be converted to UPC-E.

        The expected value of ``base`` is a 6-digit string. If any other value
        is provided, this method will use a random 6-digit string instead.

        The expected value of ``number_system_digit`` is the integer ``0`` or
        ``1``. If any other value is provided, this method will randomly choose
        from the two.

        Please also view notes on |EnUsBarcodeProvider.upc_a| and
        |EnUsBarcodeProvider.upc_e| for more details.
        """
        base_ = (
            [int(x) for x in base]
            if isinstance(base, str) and self.upc_e_base_pattern.match(base)
            else [self.random_int(0, 9) for _ in range(6)]
        )
        if number_system_digit not in [0, 1]:
            number_system_digit = self.random_int(0, 1)

        if base_[-1] <= 2:
            code = base_[:2] + base_[-1:] + [0] * 4 + base_[2:-1]
        elif base_[-1] <= 4:
            code = base_[: base_[-1]] + [0] * 5 + base_[base_[-1] : -1]
        else:
            code = base_[:5] + [0] * 4 + base_[-1:]

        code.insert(0, number_system_digit)
        weights = [3, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3]
        weighted_sum = sum(x * y for x, y in zip(code, weights))
        check_digit = (10 - weighted_sum % 10) % 10
        code.append(check_digit)
        return "".join(str(x) for x in code)

    def upc_a(
        self,
        upc_ae_mode: bool = False,
        base: Optional[str] = None,
        number_system_digit: Optional[int] = None,
    ) -> str:
        """Generate a 12-digit UPC-A barcode.

        The value of ``upc_ae_mode`` controls how barcodes will be generated. If
        ``False`` (default), barcodes are not guaranteed to have a UPC-E
        equivalent. In this mode, the method uses |EnUsBarcodeProvider.ean13|
        under the hood, and the values of ``base`` and ``number_system_digit``
        will be ignored.

        If ``upc_ae_mode`` is ``True``, the resulting barcodes are guaranteed to
        have a UPC-E equivalent, and the values of ``base`` and
        ``number_system_digit`` will be used to control what is generated.

        Under this mode, ``base`` is expected to have a 6-digit string value. If
        any other value is supplied, a random 6-digit string will be used
        instead. As for ``number_system_digit``, the expected value is a ``0``
        or a ``1``. If any other value is provided, this method will randomly
        choose from the two.

        .. important::
           When ``upc_ae_mode`` is enabled, you might encounter instances where
           different values of ``base`` (e.g. ``'120003'`` and ``'120004'``)
           produce the same UPC-A barcode. This is normal, and the reason lies
           within the whole conversion process. To learn more about this and
           what ``base`` and ``number_system_digit`` actually represent, please
           refer to |EnUsBarcodeProvider.upc_e|.

        :sample:
        :sample: upc_ae_mode=True, number_system_digit=0
        :sample: upc_ae_mode=True, number_system_digit=1
        :sample: upc_ae_mode=True, base='123456', number_system_digit=0
        :sample: upc_ae_mode=True, base='120003', number_system_digit=0
        :sample: upc_ae_mode=True, base='120004', number_system_digit=0
        """
        if upc_ae_mode is True:
            return self._upc_ae(base=base, number_system_digit=number_system_digit)
        else:
            ean13 = self.ean13(leading_zero=True)
            return ean13[1:]

    def upc_e(
        self,
        base: Optional[str] = None,
        number_system_digit: Optional[int] = None,
        safe_mode: bool = True,
    ) -> str:
        """Generate an 8-digit UPC-E barcode.

        UPC-E barcodes can be expressed in 6, 7, or 8-digit formats, but this
        method uses the 8 digit format, since it is trivial to convert to the
        other two formats. The first digit (starting from the left) is
        controlled by ``number_system_digit``, and it can only be a ``0`` or a
        ``1``. The last digit is the check digit that is inherited from the
        UPC-E barcode's UPC-A equivalent. The middle six digits are collectively
        referred to as the ``base`` (for a lack of a better term).

        On that note, this method uses ``base`` and ``number_system_digit`` to
        first generate a UPC-A barcode for the check digit, and what happens
        next depends on the value of ``safe_mode``. The argument ``safe_mode``
        exists, because there are some UPC-E values that share the same UPC-A
        equivalent. For example, any UPC-E barcode of the form ``abc0000d``,
        ``abc0003d``, and ``abc0004d`` share the same UPC-A value
        ``abc00000000d``, but that UPC-A value will only convert to ``abc0000d``
        because of (a) how UPC-E is just a zero-suppressed version of UPC-A and
        (b) the rules around the conversion.

        If ``safe_mode`` is ``True`` (default), this method performs another set
        of conversions to guarantee that the UPC-E barcodes generated can be
        converted to UPC-A, and that UPC-A barcode can be converted back to the
        original UPC-E barcode. Using the example above, even if the bases
        ``120003`` or ``120004`` are used, the resulting UPC-E barcode will
        always use the base ``120000``.

        If ``safe_mode`` is ``False``, then the ``number_system_digit``,
        ``base``, and the computed check digit will just be concatenated
        together to produce the UPC-E barcode, and attempting to convert the
        barcode to UPC-A and back again to UPC-E will exhibit the behavior
        described above.

        :sample:
        :sample: base='123456'
        :sample: base='123456', number_system_digit=0
        :sample: base='123456', number_system_digit=1
        :sample: base='120000', number_system_digit=0
        :sample: base='120003', number_system_digit=0
        :sample: base='120004', number_system_digit=0
        :sample: base='120000', number_system_digit=0, safe_mode=False
        :sample: base='120003', number_system_digit=0, safe_mode=False
        :sample: base='120004', number_system_digit=0, safe_mode=False
        """
        if safe_mode is not False:
            upc_ae = self._upc_ae(base=base, number_system_digit=number_system_digit)
            return self._convert_upc_a2e(upc_ae)
        else:
            upc_ae = self._upc_ae(base=base, number_system_digit=number_system_digit)
            return upc_ae[0] + "".join(str(x) for x in base or "") + upc_ae[-1]



from typing import List

from ..en_US import Provider as EnUsProvider
from ..la import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``en_PH`` locale.

    This localized provider generates pseudo-Latin text when using the standard
    lorem provider methods, and the ``english_*`` methods are also provided for
    generating text in American English. Both languages are used in this locale
    for this purpose.

    All the ``english_*`` methods use their corresponding standard lorem
    provider method under the hood with ``ext_word_list`` set to the
    |EnUsLoremProvider|'s word list.

    .. |EnUsLoremProvider| replace::
        :meth:`EnUsLoremProvider <faker.providers.lorem.en_US.Provider>`
    """

    english_word_list = EnUsProvider.word_list

    def english_word(self) -> str:
        """Generate an English word."""
        return self.word(ext_word_list=self.english_word_list)

    def english_words(self, nb: int = 3, unique: bool = False) -> List[str]:
        """Generate a list of English words.

        :sample: nb=5
        :sample: nb=5, unique=True
        """
        return self.words(nb=nb, ext_word_list=self.english_word_list, unique=unique)

    def english_sentence(self, nb_words: int = 6, variable_nb_words: bool = True) -> str:
        """Generate a sentence in English.

        :sample: nb_words=10
        :sample: nb_words=10, variable_nb_words=False
        """
        return self.sentence(nb_words, variable_nb_words, self.english_word_list)

    def english_sentences(self, nb: int = 3) -> List[str]:
        """Generate a list of sentences in English.

        :sample: nb=5
        """
        return self.sentences(nb, self.english_word_list)

    def english_paragraph(self, nb_sentences: int = 3, variable_nb_sentences: bool = True) -> str:
        """Generate a paragraph in English.

        :sample: nb_sentences=5
        :sample: nb_sentences=5, variable_nb_sentences=False
        """
        return self.paragraph(nb_sentences, variable_nb_sentences, self.english_word_list)

    def english_paragraphs(self, nb: int = 3) -> List[str]:
        """Generate a list of paragraphs in English.

        :sample: nb=5
        """
        return self.paragraphs(nb, self.english_word_list)

    def english_text(self, max_nb_chars: int = 200) -> str:
        """Generate a text string in English.

        :sample: max_nb_chars=20
        :sample: max_nb_chars=80
        :sample: max_nb_chars=160
        """
        return self.text(max_nb_chars, self.english_word_list)

    def english_texts(self, nb_texts: int = 3, max_nb_chars: int = 200) -> List[str]:
        """Generate a list of text strings in English.

        :sample: nb_texts=5
        :sample: nb_texts=5, max_nb_chars=50
        """
        return self.texts(nb_texts, max_nb_chars, self.english_word_list)



"""
This module exists solely to figure how long a registrant/publication
number may be within an ISBN. The rules change based on the prefix and
language/region. This list of rules only encapsulates the 978 prefix
for English books. 978 is the largest and, until recently, the only
prefix.

The complete list of prefixes and rules can be found at
https://www.isbn-international.org/range_file_generation
"""

from collections import namedtuple
from typing import Dict, List

RegistrantRule = namedtuple("RegistrantRule", ["min", "max", "registrant_length"])

# Structure: RULES[`EAN Prefix`][`Registration Group`] = [Rule1, Rule2, ...]
RULES: Dict[str, Dict[str, List[RegistrantRule]]] = {
    "978": {
        "0": [
            RegistrantRule("0000000", "1999999", 2),
            RegistrantRule("2000000", "2279999", 3),
            RegistrantRule("2280000", "2289999", 4),
            RegistrantRule("2290000", "6479999", 3),
            RegistrantRule("6480000", "6489999", 7),
            RegistrantRule("6490000", "6999999", 3),
            RegistrantRule("7000000", "8499999", 4),
            RegistrantRule("8500000", "8999999", 5),
            RegistrantRule("9000000", "9499999", 6),
            RegistrantRule("9500000", "9999999", 7),
        ],
        "1": [
            RegistrantRule("0000000", "0999999", 2),
            RegistrantRule("1000000", "3999999", 3),
            RegistrantRule("4000000", "5499999", 4),
            RegistrantRule("5500000", "7319999", 5),
            RegistrantRule("7320000", "7399999", 7),
            RegistrantRule("7400000", "8697999", 5),
            RegistrantRule("8698000", "9729999", 6),
            RegistrantRule("9730000", "9877999", 4),
            RegistrantRule("9878000", "9989999", 6),
            RegistrantRule("9990000", "9999999", 7),
        ],
    },
}



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    DAY_NAMES = {
        "0": "domingo",
        "1": "lunes",
        "2": "martes",
        "3": "miércoles",
        "4": "jueves",
        "5": "viernes",
        "6": "sábado",
    }

    MONTH_NAMES = {
        "01": "enero",
        "02": "febrero",
        "03": "marzo",
        "04": "abril",
        "05": "mayo",
        "06": "junio",
        "07": "julio",
        "08": "agosto",
        "09": "septiembre",
        "10": "octubre",
        "11": "noviembre",
        "12": "diciembre",
    }

    def day_of_week(self):
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



from typing import Dict

from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``pl_PL`` locale.

    Source: https://pl.wiktionary.org/wiki/Indeks%3APolski_-_Najpopularniejsze_s%C5%82owa_1-2000
    """

    word_list = (
        "w",
        "z",
        "być",
        "na",
        "i",
        "do",
        "nie",
        "który",
        "lub",
        "to",
        "się",
        "o",
        "mieć",
        "coś",
        "ten",
        "dotyczyć",
        "on",
        "od",
        "co",
        "język",
        "po",
        "że",
        "ktoś",
        "przez",
        "osoba",
        "miasto",
        "jeden",
        "jak",
        "za",
        "ja",
        "rok",
        "a",
        "bardzo",
        "swój",
        "dla",
        "taki",
        "człowiek",
        "cecha",
        "kobieta",
        "mój",
        "część",
        "związany",
        "móc",
        "dwa",
        "ona",
        "związać",
        "ze",
        "mały",
        "jakiś",
        "miejsce",
        "inny",
        "duży",
        "bez",
        "czas",
        "ale",
        "czy",
        "jako",
        "sposób",
        "rodzaj",
        "Polska",
        "rodzina",
        "tylko",
        "mieszkaniec",
        "dzień",
        "praca",
        "przed",
        "dom",
        "dziecko",
        "ty",
        "pod",
        "tak",
        "woda",
        "np.",
        "już",
        "rzeka",
        "zostać",
        "dobry",
        "życie",
        "państwo",
        "mówić",
        "pierwszy",
        "nasz",
        "cały",
        "nad",
        "wiele",
        "zwierzę",
        "przy",
        "roślina",
        "ta",
        "u",
        "jego",
        "gatunek",
        "nowy",
        "chcieć",
        "sobie",
        "wielki",
        "często",
        "trzy",
        "kolor",
        "używać",
        "musieć",
        "kraj",
        "robić",
        "strona",
        "każdy",
        "wysoki",
        "nazwa",
        "mężczyzna",
        "grupa",
        "my",
        "stary",
        "sam",
        "stan",
        "drugi",
        "zrobić",
        "iść",
        "oraz",
        "polski",
        "litera",
        "kto",
        "prawo",
        "drzewo",
        "ptak",
        "książka",
        "świat",
        "samochód",
        "rzecz",
        "stolica",
        "między",
        "droga",
        "należeć",
        "mieszkanka",
        "słowo",
        "gdy",
        "głowa",
        "pies",
        "młody",
        "symbol",
        "oni",
        "bo",
        "ziemia",
        "aby",
        "owoc",
        "liczba",
        "wiek",
        "nie-",
        "kilka",
        "zły",
        "środek",
        "znajdować się",
        "raz",
        "dobrze",
        "pan",
        "kiedy",
        "okres",
        "pochodzić",
        "ojciec",
        "długi",
        "ręka",
        "itp.",
        "odnosić się",
        "dużo",
        "podczas",
        "biały",
        "albo",
        "ruch",
        "jaki",
        "przedmiot",
        "służyć",
        "matka",
        "we",
        "znak",
        "ci",
        "siebie",
        "liczba atomowa",
        "jeszcze",
        "niż",
        "cztery",
        "wszystko",
        "widzieć",
        "żona",
        "koń",
        "szkoła",
        "ciało",
        "stać",
        "kupić",
        "zawsze",
        "forma",
        "sprawa",
        "Rosja",
        "wieś",
        "góra",
        "wyspa",
        "oko",
        "działanie",
        "twój",
        "występować",
        "koniec",
        "rząd",
        "pięć",
        "pokój",
        "nauka",
        "gdzie",
        "kwiat",
        "choroba",
        "zwykle",
        "powiedzieć",
        "mieszkać",
        "wiedzieć",
        "imię",
        "prowadzić",
        "element",
        "dać",
        "godzina",
        "żyć",
        "ryba",
        "wszyscy",
        "zawierać",
        "pracować",
        "by",
        "alfabet",
        "członek",
        "syn",
        "jednostka",
        "herb",
        "brat",
        "las",
        "urządzenie",
        "miesiąc",
        "dziewczyna",
        "obszar",
        "grać",
        "różny",
        "teren",
        "piękny",
        "jeść",
        "nic",
        "brak",
        "żeby",
        "lubić",
        "dany",
        "budynek",
        "położyć",
        "czerwony",
        "cel",
        "stopień",
        "siła",
        "światło",
        "leżeć",
        "dawać",
        "gra",
        "sztuka",
        "czarny",
        "one",
        "jej",
        "wino",
        "chodzić",
        "statek",
        "krótki",
        "śmierć",
        "wartość",
        "dźwięk",
        "sytuacja",
        "teraz",
        "główny",
        "zajmować się",
        "wykonywać",
        "związek",
        "ważny",
        "ostatni",
        "1000",
        "tam",
        "noc",
        "dziś",
        "pierwiastek chemiczny",
        "wojna",
        "noga",
        "sklep",
        "skóra",
        "pani",
        "własny",
        "materiał",
        "niektóry",
        "tworzyć",
        "system",
        "znany",
        "także",
        "wykonać",
        "niebo",
        "święty",
        "władza",
        "wczoraj",
        "film",
        "twarz",
        "flaga",
        "morze",
        "nawet",
        "mięso",
        "głos",
        "Europa",
        "?",
        "pieniądz",
        "powierzchnia",
        "proces",
        "tydzień",
        "posiadać",
        "ilość",
        "obwód",
        "działać",
        "północny",
        "region",
        "jeśli",
        "trwać",
        "szybko",
        "Bóg",
        "silny",
        "!",
        "lecz",
        "zielony",
        "określony",
        "król",
        "pole",
        "przyjaciel",
        "1",
        "dwadzieścia",
        "serce",
        "sześć",
        "słońce",
        "pisać",
        "kot",
        "drzwi",
        "znać",
        "początek",
        "tysiąc",
        "mleko",
        "południowy",
        "obraz",
        "nosić",
        "wiatr",
        "niski",
        "tekst",
        "pić",
        "zmiana",
        "dawny",
        "ulica",
        "kierunek",
        "linia",
        "jechać",
        "wyraz",
        "stanowić",
        "charakterystyczny",
        "składać się",
        "tu",
        "uważać",
        "siedem",
        "miłość",
        "podobny",
        "więc",
        "żołnierz",
        "siostra",
        "córka",
        "też",
        "chleb",
        "zacząć",
        "koło",
        "granica",
        "powietrze",
        "pewien",
        "włos",
        "charakter",
        "punkt",
        "dzisiaj",
        "ludzie",
        "mało",
        "liść",
        "(…)",
        "znaleźć",
        "kościół",
        "badanie",
        "niewielki",
        "wziąć",
        "prosty",
        "krew",
        "mąż",
        "–",
        "wolny",
        "kawa",
        "problem",
        "pójść",
        "powodować",
        "czyjś",
        "drewno",
        "kształt",
        "stać się",
        "właściwy",
        "trzeci",
        "znaczenie",
        "brzeg",
        "historia",
        "ich",
        "zasada",
        "brać",
        "dziesięć",
        "powinien",
        "żaden",
        "jezioro",
        "okno",
        "kultura",
        "niemiecki",
        "ostry",
        "but",
        "stosować",
        "ogień",
        "nigdy",
        "zbiór",
        "samolot",
        "ból",
        "osiem",
        "można",
        "gwiazda",
        "walka",
        "Ukraina",
        "prawdziwy",
        "ciężki",
        "zespół",
        "drogi",
        "pracownik",
        "Francja",
        "myśleć",
        "zachowanie",
        "polegać",
        "uwaga",
        "pomoc",
        "przypominać",
        "grecki",
        "Niemcy",
        "ząb",
        "ile",
        "informacja",
        "chwila",
        "deszcz",
        "istnieć",
        "nauczyciel",
        "żółty",
        "chory",
        "piwo",
        "według",
        "dostać",
        "uczeń",
        "jedzenie",
        "śnieg",
        "jednak",
        "również",
        "ani",
        "zwłaszcza",
        "utwór",
        "czysty",
        "firma",
        "siedzieć",
        "francuski",
        "łączyć",
        "południe",
        "zbyt",
        "trudny",
        "urząd",
        "stół",
        "lekarz",
        "muzyka",
        "czynność",
        "układ okresowy",
        "pociąg",
        "jasny",
        "klasa",
        "męski",
        "kamień",
        "pierwiastek",
        "ubranie",
        "ściana",
        "postać",
        "pełny",
        "organizm",
        "5",
        "księżyc",
        "gmina",
        "rosnąć",
        "w celu",
        "wydawać",
        "źródło",
        "funkcja",
        "położenie",
        "typ",
        "starożytny",
        "jutro",
        "dziewięć",
        "trzeba",
        "społeczny",
        "prawy",
        "program",
        "pojazd",
        "może",
        "historyczny",
        "2",
        "substancja",
        "wszystkie",
        "piec",
        "układ",
        "bóg",
        "polityczny",
        "chłopiec",
        "cena",
        "słaby",
        "głupi",
        "ludzki",
        "trzymać",
        "zupa",
        "około",
        "mieszkanie",
        "zdanie",
        "naczynie",
        "uprawiać",
        "północ",
        "kraina",
        "numer",
        "para",
        "dokument",
        "uczucie",
        "prawda",
        "złoty",
        "za pomocą",
        "elektryczny",
        "dziedzina",
        "zachodni",
        "alkohol",
        "trochę",
        "prowincja",
        "prosić",
        "list",
        "bliski",
        "komputer",
        "towar",
        "szybki",
        "spać",
        "niebieski",
        "aż",
        "przypadek",
        "organizacja",
        "herbata",
        "szeroki",
        "kawałek",
        "czytać",
        "obejmować",
        "wojskowy",
        "narzędzie",
        "przyjść",
        "myśl",
        "ogród",
        "Włochy",
        "całość",
        "wieczór",
        "lód",
        "wiedza",
        "powiat",
        "połowa",
        "angielski",
        "głównie",
        "zjawisko",
        "chłopak",
        "wpływ",
        "mowa",
        "naturalny",
        "morski",
        "produkt",
        "lewy",
        "prawie",
        "lek",
        "miejscowość",
        "napój",
        "wschodni",
        "księga",
        "stopa",
        "drobny",
        "ciasto",
        "kuchnia",
        "plan",
        "powstać",
        "pełen",
        "wokół",
        "kochać",
        "palec",
        "zobaczyć",
        "poprzez",
        "maszyna",
        "dziadek",
        "wielkość",
        "nos",
        "złoto",
        "pewny",
        "partia",
        "większość",
        "obiekt",
        "publiczny",
        "pismo",
        "wybitny",
        "wszystek",
        "błąd",
        "broń",
        "sen",
        "trzydzieści",
        "gruby",
        "spotkanie",
        "tkanina",
        "smak",
        "gość",
        "potrawa",
        "pytanie",
        "produkcja",
        "wy",
        "razem",
        "obywatel",
        "jajko",
        "3",
        "zima",
        "nazywać",
        "policja",
        "nikt",
        "słodki",
        "dopływ",
        "butelka",
        "energia",
        "składać",
        "łóżko",
        "urodzenie",
        "zdrowie",
        "odmiana",
        "zdjęcie",
        "mocny",
        "poza",
        "4",
        "lekki",
        "czynić",
        "przeciwny",
        "duch",
        "sąd",
        "przeznaczyć",
        "zapach",
        "stały",
        "Afryka",
        "styl",
        "karta",
        "wypadek",
        "babcia",
        "wojsko",
        "wodny",
        "równy",
        "rola",
        "rejon",
        "wybrzeże",
        "naród",
        "wiadomość",
        "kość",
        "tytuł",
        "cukier",
        "barwa",
        "żywy",
        "szczyt",
        "rozwój",
        "sieć",
        "30",
        "ponad",
        "lato",
        "warstwa",
        "jabłko",
        "wyrażać",
        "bogaty",
        "odbywać się",
        "podstawowy",
        "cześć",
        "z powodu",
        "ponieważ",
        "wyjść",
        "poziom",
        "wyglądać",
        "śpiewać",
        "oznaczać",
        "rozmowa",
        "ciemny",
        "papier",
        "900",
        "palić",
        "lud",
        "długość",
        "usta",
        "ucho",
        "urodzić",
        "wewnątrz",
        "wśród",
        "przedstawiciel",
        "środkowy",
        "obok",
        "dzieło",
        "arabski",
        "krowa",
        "taniec",
        "rano",
        "grzyb",
        "długo",
        "wydarzenie",
        "pięćdziesiąt",
        "włoski",
        "słuchać",
        "ser",
        "właśnie",
        "stanowisko",
        "odpowiedni",
        "korona",
        "rower",
        "święto",
        "czekać",
        "szukać",
        "100",
        "religia",
        "piłka",
        "opinia",
        "wynik",
        "pozycja",
        "pochodzenie",
        "metoda",
        "ciepły",
        "potem",
        "udział",
        "Hiszpania",
        "rozumieć",
        "6",
        "wspólny",
        "środowisko",
        "całkowicie",
        "budowa",
        "ramię",
        "gazeta",
        "zabawa",
        "nie ma",
        "szczęście",
        "pomieszczenie",
        "strach",
        "fala",
        "patrzeć",
        "odcień",
        "temperatura",
        "warunek",
        "zdolność",
        "sól",
        "rosyjski",
        "podróż",
        "wykorzystywać",
        "Ziemia",
        "religijny",
        "centrum",
        "zbierać",
        "zupełnie",
        "przestrzeń",
        "pas",
        "połączenie",
        "wobec",
        "stawać się",
        "potrzeba",
        "narodowy",
        "liczyć",
        "otwarty",
        "wejść",
        "pozbawić",
        "masa",
        "głęboki",
        "ono",
        "wywoływać",
        "zachód",
        "wschód",
        "powód",
        "Azja",
        "administracyjny",
        "temat",
        "odpowiadać",
        "szpital",
        "zajmować",
        "czterdzieści",
        "sto",
        "sobą",
        "pogląd",
        "chronić",
        "wysokość",
        "słownik",
        "rodzic",
        "świnia",
        "zaczynać",
        "moneta",
        "możliwość",
        "mama",
        "gdzieś",
        "egzamin",
        "pogoda",
        "chemiczny",
        "gorący",
        "zadanie",
        "więzienie",
        "zakład",
        "ofiara",
        "obiad",
        "wąski",
        "zamek",
        "moc",
        "stosunek",
        "natura",
        "8",
        "zazwyczaj",
        "założyć",
        "skrzydło",
        "otrzymać",
        "oficjalny",
        "chmura",
        "ten sam",
        "złożyć",
        "wewnętrzny",
        "wspaniały",
        "przyczyna",
        "miły",
        "dziki",
        "kara",
        "listopad",
        "komórka",
        "instytucja",
        "skała",
        "ogromny",
        "wygląd",
        "sześćdziesiąt",
        "możliwy",
        "wąż",
        "umrzeć",
        "określać",
        "amerykański",
        "płynąć",
        "walczyć",
        "nóż",
        "nagle",
        "instrument",
        "20",
        "rynek",
        "Grecja",
        "umowa",
        "niedziela",
        "szczęśliwy",
        "tutaj",
        "zmieniać",
        "węgiel",
        "sylaba",
        "Warszawa",
        "ładny",
        "europejski",
        "czwarty",
        "styczeń",
        "hiszpański",
        "posługiwać się",
        "papieros",
        "fizyczny",
        "dach",
        "zimny",
        "ogon",
        "trawa",
        "telefon",
        "płyn",
        "przedstawiać",
        "metal",
        "dlaczego",
        "próbować",
        "10",
        "7",
        "sportowy",
        "oddział",
        "obecnie",
        "9",
        "miara",
        "prezydent",
        "pierś",
        "rodowity",
        "stworzyć",
        "dział",
        "dusza",
        "wierzyć",
        "domowy",
        "właściciel",
        "wyrób",
        "autobus",
        "ponownie",
        "gaz",
        "właściwość",
        "rada",
        "rzymski",
        "bieg",
        "zgoda",
        "obowiązek",
        "owca",
        "zamieszkiwać",
        "przyjąć",
        "muzyczny",
        "przyrząd",
        "piąty",
        "szczególnie",
        "kupować",
        "istota",
        "stracić",
        "artykuł",
        "ochrona",
        "te",
        "napisać",
        "specjalista",
        "ku",
        "górski",
        "należy",
        "określenie",
        "pomiędzy",
        "Rzym",
        "ssak",
        "zwolennik",
        "odpowiedź",
        "działalność",
        "miejski",
        "wcześnie",
        "zdobyć",
        "górny",
        "uniwersytet",
        "bić",
        "wymagać",
        "miękki",
        "źle",
        "40",
        "państwowy",
        "ludność",
        "minuta",
        "cierpieć",
        "ogół",
        "naprawdę",
        "blisko",
        "surowy",
        "dodatek",
        "radość",
        "akcja",
        "w kształcie",
        "polityka",
        "obcy",
        "ziemniak",
        "podstawa",
        "przemysł",
        "udać się",
        "brzuch",
        "suchy",
        "krzew",
        "terytorium",
        "wolność",
        "czyli",
        "klucz",
        "Jan",
        "kolejny",
        "uczyć się",
        "postępowanie",
        "sok",
        "50",
        "łatwo",
        "jeździć",
        "decyzja",
        "naukowy",
        "szanowny",
        "warzywo",
        "nadzieja",
        "wrzesień",
        "kierować",
        "student",
        "kąt",
        "seksualny",
        "piasek",
        "drewniany",
        "obchodzić",
        "wróg",
        "przeciwko",
        "żeński",
        "potrafić",
        "pamięć",
        "teatr",
        "dwudziesty",
        "znowu",
        "potrzebować",
        "owad",
        "cienki",
        "ziarno",
        "moment",
        "wiosna",
        "wydać",
        "literatura",
        "tradycyjny",
        "leczenie",
        "poważny",
        "siedemdziesiąt",
        "silnik",
        "spokój",
        "luty",
        "biedny",
        "czuć",
        "drużyna",
        "dialekt",
        "dzięki",
        "grudzień",
        "jedyny",
        "pragnienie",
        "siedziba",
        "służba",
        "wiara",
        "pióro",
        "wzrost",
        "proszę",
        "osiemdziesiąt",
        "społeczeństwo",
        "dokładnie",
        "przykład",
        "szacunek",
        "marzec",
        "róg",
        "połączyć",
        "uderzenie",
        "zwyczaj",
        "podawać",
        "mocno",
        "zwykły",
        "kolega",
        "międzynarodowy",
        "sala",
        "nadawać",
        "tamten",
        "szósty",
        "lekcja",
        "pomagać",
        "republika",
        "zjeść",
        "typowy",
        "modlitwa",
        "dół",
        "dlatego",
        "rasa",
        "użycie",
        "dziewięćdziesiąt",
        "bok",
        "zatoka",
        "wiersz",
        "Szwecja",
        "japoński",
        "gałąź",
        "wrogi",
        "przyjmować",
        "więcej",
        "łatwy",
        "atak",
        "wychodzić",
        "wtedy",
        "płyta",
        "milion",
        "padać",
        "kanał",
        "poniedziałek",
        "wzór",
        "twardy",
        "podatek",
        "rzucać",
        "świeży",
        "bilet",
        "zakładać",
        "złapać",
        "przyszłość",
        "przyjęcie",
        "zewnętrzny",
        "zamknąć",
        "przynosić",
        "obecny",
        "strój",
        "popularny",
        "późno",
        "płaski",
        "struktura",
        "pieniądze",
        "projekt",
        "doświadczenie",
        "szyja",
        "rozmawiać",
        "literacki",
        "okolica",
        "mur",
        "małżeństwo",
        "bitwa",
        "kwiecień",
        "maj",
        "specjalny",
        "poruszać się",
        "sąsiad",
        "organ",
        "pamiętać",
        "uczyć",
        "termin",
        "bank",
        "pusty",
        "pół",
        "wchodzić",
        "czyn",
        "Japonia",
        "przeciw",
        "wczesny",
        "wejście",
        "ciągle",
        "bać się",
        "Stany Zjednoczone",
        "delikatny",
        "wilk",
        "kula",
        "r.",
        "wnętrze",
        "prąd",
        "sprzedawać",
        "port",
        "spokojny",
        "waga",
        "sztuczny",
        "Polak",
        "jajo",
        "dym",
        "pszczoła",
        "technika",
        "współczesny",
        "widoczny",
        "krok",
        "próba",
        "gęsty",
        "miód",
        "ciepło",
        "mapa",
        "kapelusz",
        "otwór",
        "lew",
        "osioł",
        "gwałtowny",
        "siódmy",
        "budować",
        "los",
        "telewizja",
        "spowodować",
        "dłoń",
        "akt",
        "mysz",
        "jesień",
        "składnik",
        "słyszeć",
        "zgodnie",
        "zdrowy",
        "masło",
        "gwiazdozbiór",
        "kino",
        "podać",
        "zmienić",
        "przechodzić",
        "fabryka",
        "dość",
        "daleko",
        "z krwi i kości",
        "płeć",
        "chiński",
        "sędzia",
        "pokryć",
        "lotniczy",
        "nazwisko",
        "bądź",
        "Słowacja",
        "umieć",
        "majątek",
        "ocena",
        "pływać",
        "komputerowy",
        "dziać się",
        "ósmy",
        "autor",
        "sierpień",
        "łagodny",
        "zakres",
        "wybrać",
        "następny",
        "odległość",
        "most",
        "policjant",
        "panować",
        "zawód",
        "zwrot",
        "wybór",
        "Chiny",
        "internetowy",
        "wytwarzać",
        "lipiec",
        "bohater",
        "prasa",
        "penis",
        "Czechy",
        "80",
        "fakt",
        "piosenka",
        "mąka",
        "badać",
        "sobota",
        "piątek",
        "znajdować",
        "straszny",
        "waluta",
        "pojechać",
        "otwierać",
        "umiejętność",
        "ślub",
        "restauracja",
        "przedsiębiorstwo",
        "towarzystwo",
        "model",
        "no",
        "prywatny",
        "reakcja",
        "okazja",
        "porządek",
        "opowiadać",
        "przeciwnik",
        "mięsień",
        "zysk",
        "sprzedaż",
        "zabić",
        "różnica",
        "klasztor",
        "osiągnąć",
        "niebezpieczny",
        "pisarz",
        "wrócić",
        "skład",
        "m.in.",
        "sprawiać",
        "chrześcijański",
        "zapomnieć",
        "gniew",
        "planeta",
        "postawić",
        "przejść",
        "kurs",
        "przygotować",
        "dzielić",
        "dzielnica",
        "kierowca",
        "własność",
        "królowa",
        "korzeń",
        "artysta",
        "stawiać",
        "jakość",
        "przyjemność",
        "średni",
        "ludowy",
        "całkowity",
        "Dania",
        "biblioteka",
        "dopiero",
        "zero",
        "gniazdo",
        "pieśń",
        "urzędnik",
        "przestać",
        "dziura",
        "Anglia",
        "mózg",
        "liczny",
        "uderzać",
        "efekt",
        "rozmiar",
        "przyjemny",
        "norma",
        "pozwalać",
        "rana",
        "korzyść",
        "tańczyć",
        "kosztować",
        "Słońce",
        "podział",
        "samica",
        "przepis",
        "hotel",
        "rzadko",
        "wykonanie",
        "brzydki",
        "otworzyć",
        "armia",
        "kiedyś",
        "brązowy",
        "rzeczywistość",
        "prędkość",
        "szef",
        "ciecz",
        "kaczka",
        "szkolny",
        "dokonywać",
        "fałszywy",
        "koszula",
        "tyle",
        "rzeczownik",
        "złożony",
        "zawodnik",
        "-",
        "tradycja",
        "śniadanie",
        "usługa",
        "skończyć",
        "Białoruś",
        "znów",
        "handel",
        "mieć na imię",
        "królestwo",
        "jądro",
        "powstawać",
        "okrągły",
        "spodnie",
        "powoli",
        "godny",
        "jeżeli",
        "ślad",
        "przedstawienie",
        "olej",
        "jazda",
        "dyskusja",
        "wyrażenie",
        "daleki",
        "sądzić",
        "Ameryka",
        "tracić",
        "znosić",
        "profesor",
        "świątynia",
        "szary",
        "piłka nożna",
        "zboże",
        "uderzyć",
        "wola",
        "srebro",
        "dolina",
        "w postaci",
        "różowy",
        "zamykać",
        "wrogość",
        "Indie",
        "dziwny",
        "czasem",
        "temu",
        "wtorek",
        "oglądać",
        "sport",
        "małpa",
        "spotkać",
        "zdarzenie",
        "wódka",
        "wrażenie",
        "kalendarz",
        "pomysł",
        "odczuwać",
        "koszt",
        "plemię",
        "bydło",
        "strumień",
        "skutek",
        "książę",
        "całkiem",
        "papież",
        "dodawać",
        "brudny",
        "przyszły",
        "mecz",
        "scena",
        "wolno",
        "klient",
        "opisać",
        "szereg",
        "ciąża",
        "coraz",
        "złodziej",
        "Izrael",
        "głód",
        "otaczać",
        "władca",
        "transport",
        "w formie",
        "niebezpieczeństwo",
        "słoneczny",
        "figura",
        "wszelki",
        "wysiłek",
        "kolano",
        "niech",
        "tłuszcz",
        "zakończenie",
        "mi",
        "ksiądz",
        "żelazo",
        "łuk",
        "mebel",
        "Afganistan",
        "nieszczęście",
        "wskazywać",
        "plaża",
        "fragment",
        "zaś",
        "metr",
        "kościelny",
        "samochodowy",
        "zachowywać się",
        "obrona",
        "danie",
        "wierny",
        "amharski",
        "lista",
        "żart",
        "ogólny",
        "kontrola",
        "budzić",
        "90",
        "tłum",
        "naj-",
        "kontakt",
        "czasownik",
        "gotowy",
        "Jezus",
        "koza",
        "zbiornik",
        "obserwować",
        "grób",
        "stacja",
        "robotnik",
        "czerwiec",
        "październik",
        "konstrukcja",
        "choć",
        "wyjście",
        "minerał",
        "kosz",
        "60",
        "cebula",
        "samiec",
        "sos",
        "zmarły",
        "ojczyzna",
        "bycie",
        "szkoda",
        "niszczyć",
        "majuskuła",
        "przejaw",
        "zniszczyć",
        "niedźwiedź",
        "pokazywać",
        "gospodarka",
        "zbudować",
        "dodatkowy",
        "park",
        "opłata",
        "wysoko",
        "Egipt",
        "zegar",
        "wujek",
        "dawno",
        "studia",
        "cesarz",
        "wizyta",
        "przyprawa",
        "łódź",
        "powszechny",
        "robota",
        "metalowy",
        "biec",
        "dobro",
        "dzisiejszy",
        "obóz",
        "żydowski",
        "USA",
        "Chrystus",
        "oddawać",
        "widok",
        "marka",
        "pojęcie",
        "miecz",
        "krzyż",
        "tajemnica",
        "chłop",
        "Austria",
        "lecieć",
        "bezpieczeństwo",
        "królewski",
        "śmiech",
        "postawa",
        "sukces",
        "zgodny",
        "płaszcz",
        "Turcja",
        "przeszkoda",
        "prostytutka",
        "operacja",
        "wywołać",
        "narząd",
        "futro",
        "świeca",
        "Australia",
        "prawny",
        "wciąż",
        "Szwajcaria",
        "powieść",
        "gotować",
        "szczególny",
        "rozwiązanie",
        "relacja",
        "studiować",
        "stado",
        "w czasie",
        "kontynent",
        "przychodzić",
        "lis",
        "strefa",
        "70",
        "wypowiedź",
        "dziewiąty",
        "idea",
        "kura",
        "grunt",
        "farba",
        "wóz",
        "epoka",
        "lęk",
        "smutny",
        "kolejowy",
        "dodać",
        "uchodzić",
        "przygotowywać",
        "przynieść",
        "umysł",
        "suma",
        "interes",
        "produkować",
        "Boże Narodzenie",
        "wieża",
        "handlowy",
        "gdyby",
        "Kraków",
        "utrzymywać",
        "urodziny",
        "natychmiast",
        "uciekać",
        "chociaż",
        "słoń",
        "prezent",
        "odwaga",
        "ciężar",
        "płacić",
        "podłoga",
        "atmosfera",
        "wspólnota",
        "zwycięstwo",
        "treść",
        "zainteresowanie",
        "zamiast",
        "tor",
        "artystyczny",
        "dwanaście",
        "zdolny",
        "pojedynczy",
        "przejście",
        "moralny",
        "reguła",
        "naukowiec",
        "osobisty",
        "mnóstwo",
        "wybory",
        "jedynie",
        "wada",
        "sygnał",
        "wykonywanie",
        "wybierać",
        "umieszczać",
        "mistrz",
        "nagły",
        "dno",
        "pomarańczowy",
        "telewizyjny",
        "radio",
        "przerwa",
        "matematyka",
        "klub",
        "środa",
        "muzeum",
        "finansowy",
        "malować",
        "opieka",
        "Żyd",
        "ośrodek",
        "krzesło",
        "ukraiński",
        "kolej",
        "kłopot",
        "ryż",
        "cień",
        "szwedzki",
        "usuwać",
        "katolicki",
        "cierpienie",
        "znaczny",
        "umożliwiać",
        "Rumunia",
        "poznać",
        "wynosić",
        "pijany",
        "zakończyć",
        "intensywny",
        "kostka",
        "świadczyć",
        "wydawać się",
        "godność",
        "Unia Europejska",
        "orzeł",
        "burza",
        "chrześcijaństwo",
        "błoto",
        "biskup",
        "gardło",
        "szkło",
        "polityk",
        "umieścić",
        "pozostać",
        "czwartek",
        "piętro",
        "odkryć",
        "powstanie",
        "zakon",
        "oddech",
        "nastrój",
        "teoria",
        "doskonały",
        "dolny",
        "spadek",
        "zawartość",
        "zatrzymać",
        "aktor",
        "grzech",
        "otrzymywać",
        "anioł",
        "szklanka",
        "ciekawy",
        "pomóc",
        "pomidor",
        "smutek",
        "Wielka Brytania",
        "pora",
        "śmiać się",
        "abugida",
        "odcinek",
        "nasiono",
        "pokarm",
        "zimno",
        "wieczorem",
        "wracać",
        "azjatycki",
        "wysłać",
        "sprzęt",
        "posiłek",
        "ozdobny",
        "impreza",
        "potrzebny",
        "znaczyć",
        "łyżka",
        "narkotyk",
        "biuro",
        "parlament",
        "obywatelka",
        "babka",
        "zabawka",
        "dorosły",
        "ćwiczenie",
        "ocean",
        "nadmierny",
        "niezwykły",
        "bieda",
        "użytkownik",
        "polować",
        "dyrektor",
        "procent",
        "ziemski",
        "spór",
        "żaba",
        "starać się",
        "w wyniku",
        "pacjent",
        "Litwa",
        "wycieczka",
        "istotny",
        "lampa",
        "mgła",
        "Węgry",
        "późny",
        "dziewczynka",
        "lina",
        "w ciągu",
        "mocz",
        "motyl",
        "półwysep",
        "staw",
        "przybyć",
        "duński",
        "nieprzyjemny",
        "wakacje",
        "przestępstwo",
        "centralny",
        "odzież",
        "głośny",
        "wysyłać",
        "wina",
        "pożar",
        "pasek",
        "przyjaźń",
        "koncert",
        "zarówno",
        "turecki",
        "na zewnątrz",
        "kilometr",
        "zapalenie",
        "tani",
        "pytać",
        "św.",
        "dane",
        "poeta",
        "łąka",
        "trudność",
        "ciotka",
        "seks",
        "bar",
        "pasmo",
        "zaraz",
        "ubogi",
        "po prostu",
        "igła",
        "cmentarz",
        "dziób",
        "róża",
        "pozostawać",
        "zawodowy",
        "tablica",
        "klimat",
        "cisza",
        "okropny",
        "włosy",
        "wzdłuż",
        "medycyna",
        "bawić się",
        "wzrok",
        "w.",
        "bogini",
        "wioska",
        "letni",
        "chyba",
        "poczta",
        "deska",
        "hodować",
        "wreszcie",
        "przyjechać",
        "filmowy",
        "kończyć",
        "psychiczny",
        "uzyskać",
        "rachunek",
        "minister",
        "dowód",
        "lata",
        "mrówka",
        "radiowy",
        "średniowieczny",
        "mądry",
        "przeprowadzać",
        "kolacja",
        "jakby",
        "pragnąć",
        "sądowy",
        "ustawa",
        "zaufanie",
        "wojenny",
        "obowiązywać",
        "promień",
        "Kościół",
        "dać się",
        "kult",
        "traktować",
        "czapka",
        "ciągnąć",
        "paliwo",
        "—",
        "diabeł",
        "Holandia",
        "broda",
        "w końcu",
        "powolny",
        "muzyk",
        "korzystać",
        "sowa",
        "dokładny",
        "czoło",
        "zając",
        "na przykład",
        "płakać",
        "podnieść",
        "wybuch",
        "spaść",
        "byk",
        "budowla",
        "zgromadzenie",
        "odważny",
        "czynnik",
        "zeszły",
        "wesoły",
        "pająk",
        "opuścić",
        "ciemność",
        "kij",
        "pałac",
        "archipelag",
        "pojawiać się",
        "panna",
        "gęś",
        "nauczycielka",
        "zajęcie",
        "trudno",
        "pustynia",
        "kieszeń",
        "fotografia",
        "tytoń",
        "upadek",
        "wyrok",
        "istnienie",
        "zanim",
        "wyścig",
        "chęć",
        "świecić",
        "częściowo",
        "dokonać",
        "żywność",
        "sukienka",
        "obrót",
        "toponim",
        "wpaść",
        "podróżować",
        "kolumna",
        "rodzinny",
        "poprzedni",
        "Niemiec",
        "pisanie",
        "oddać",
        "rzadki",
        "bułgarski",
        "otoczenie",
        "kobiecy",
        "kolorowy",
        "kartka",
        "urodzić się",
        "piętnaście",
        "uznawać",
        "okręt",
        "trzydziesty",
        "wniosek",
        "głupiec",
        "strata",
        "większy",
        "podnosić",
        "nocny",
        "wywodzić się",
        "filozofia",
        "inaczej",
        "Pan",
        "ozdoba",
        "uciec",
        "martwy",
        "hałas",
        "lotnisko",
        "tył",
        "łaciński",
        "położony",
        "pełnić",
        "kwestia",
        "tarcza",
        "0",
        "skłonność",
        "go",
        "talerz",
        "wygrać",
        "Morze Śródziemne",
        "minuskuła",
        "szlachetny",
        "poruszać",
        "jadalny",
        "jedenaście",
        "nieść",
        "szkodliwy",
        "użyć",
        "lot",
        "wystawa",
        "pokonać",
        "przebywać",
        "przeszłość",
        "adres",
        "wisieć",
        "oś",
        "zmęczony",
        "katastrofa",
        "zamiar",
        "bogactwo",
        "niechęć",
        "poduszka",
        "rak",
        "jednocześnie",
        "dziecięcy",
        "wstyd",
        "białoruski",
        "rozpocząć",
        "rzucić",
        "ulegać",
        "policzek",
        "wzgórze",
        "hasło",
        "lustro",
        "wkrótce",
        "narodowość",
        "pojawić się",
        "skala",
        "zapis",
        "stowarzyszenie",
        "zgadzać się",
        "rezultat",
        "oba",
        "przecież",
        "czeski",
        "tłumaczyć",
        "rysunek",
        "kłaść",
        "aktywny",
        "gołąb",
        "praktyka",
        "okoliczność",
        "trwały",
        "oczekiwać",
        "ryzyko",
        "dostęp",
        "wyłącznie",
        "czekolada",
        "oczywiście",
        "dalej",
        "dar",
        "włożyć",
        "zrozumieć",
        "postępować",
        "srebrny",
        "doprowadzić",
        "analiza",
        "mierzyć",
        "banknot",
        "głupota",
        "głupek",
        "słowacki",
        "plama",
        "uśmiech",
        "konflikt",
        "gleba",
        "gospodarczy",
        "plecy",
        "następować",
        "zaburzenie",
        "blady",
        "spadać",
        "plac",
        "cichy",
        "alkoholowy",
        "pomarańcza",
        "bajka",
        "wprowadzać",
        "żołądek",
        "latać",
        "niewolnik",
        "rolnik",
        "wspomnienie",
        "zająć",
        "nasienie",
        "Belgia",
        "wątpliwość",
        "bezpośrednio",
        "graniczyć",
        "gorączka",
        "bronić",
        "rządzić",
        "drapieżny",
        "pojemnik",
        "Piotr",
    )

    parts_of_speech: Dict[str, tuple] = {}



import string

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``de_DE`` locale.

    Sources:

    - http://berlin.de/daten/liste-der-kfz-kennzeichen/kfz-kennz-d.csv
    """

    license_plate_prefix = (
        "A",
        "AA",
        "AB",
        "ABI",
        "ABG",
        "AC",
        "AE",
        "AIC",
        "AK",
        "AM",
        "AN",
        "AÖ",
        "AP",
        "AS",
        "AUR",
        "AW",
        "AZ",
        "B",
        "BA",
        "BAD",
        "BAR",
        "BB",
        "BC",
        "BD",
        "BGL",
        "BI",
        "BIR",
        "BIT",
        "BK",
        "BL",
        "BLK",
        "BM",
        "BN",
        "BO",
        "BOR",
        "BOT",
        "BP",
        "BRA",
        "BRB",
        "BS",
        "BT",
        "BTF",
        "BÜS",
        "BW",
        "BWL",
        "BYL",
        "BZ",
        "C",
        "CB",
        "CE",
        "CHA",
        "CO",
        "COC",
        "COE",
        "CUX",
        "CW",
        "D",
        "DA",
        "DAH",
        "DAN",
        "DAU",
        "DBR",
        "DD",
        "DE",
        "DEG",
        "DEL",
        "DGF",
        "DH",
        "DL",
        "DLG",
        "DN",
        "Do",
        "DON",
        "DU",
        "DÜW",
        "E",
        "EA",
        "EB",
        "EBE",
        "ED",
        "EE",
        "EF",
        "EI",
        "EIC",
        "EL",
        "EM",
        "EMD",
        "EMS",
        "EN",
        "ER",
        "ERB",
        "ERH",
        "ERZ",
        "ES",
        "ESW",
        "EU",
        "F",
        "FB",
        "FD",
        "FDS",
        "FF",
        "FFB",
        "FG",
        "FL",
        "FN",
        "FO",
        "FR",
        "FRG",
        "FRI",
        "FS",
        "FT",
        "FÜ",
        "G",
        "GAP",
        "GE",
        "GER",
        "GF",
        "GG",
        "GI",
        "GL",
        "GM",
        "GÖ",
        "GP",
        "GR",
        "GRZ",
        "GS",
        "GT",
        "GTH",
        "GÜ",
        "GZ",
        "H",
        "HA",
        "HAL",
        "HAM",
        "HAS",
        "HB",
        "HBN",
        "HD",
        "HDH",
        "HE",
        "HEF",
        "HEI",
        "HEL",
        "HER",
        "HF",
        "HG",
        "HGW",
        "HH",
        "HI",
        "HL",
        "HM",
        "HN",
        "HO",
        "HOL",
        "HOM",
        "HP",
        "HR",
        "HRO",
        "HS",
        "HSK",
        "HST",
        "HU",
        "HVL",
        "HWI",
        "HX",
        "HZ",
        "IGB",
        "IK",
        "IN",
        "IZ",
        "J",
        "JL",
        "K",
        "KA",
        "KB",
        "KC",
        "KE",
        "KEH",
        "KF",
        "KG",
        "KH",
        "KI",
        "KIB",
        "KL",
        "KLE",
        "KN",
        "KO",
        "KR",
        "KS",
        "KT",
        "KU",
        "KÜN",
        "KUS",
        "KYF",
        "L",
        "LA",
        "LAU",
        "LB",
        "LD",
        "LDK",
        "LDS",
        "LER",
        "LEV",
        "LG",
        "LI",
        "LIF",
        "LIP",
        "LL",
        "LM",
        "LÖ",
        "LOS",
        "LRO",
        "LSA",
        "LSN",
        "LU",
        "LWL",
        "M",
        "MA",
        "MB",
        "MD",
        "ME",
        "MEI",
        "MG",
        "MI",
        "MIL",
        "MK",
        "MKK",
        "MM",
        "MN",
        "MOL",
        "MOS",
        "MR",
        "MS",
        "MSH",
        "MSP",
        "MST",
        "MTK",
        "MÜ",
        "MÜR",
        "MVL",
        "MYK",
        "MZ",
        "MZG",
        "N",
        "NB",
        "ND",
        "NDH",
        "NE",
        "NEA",
        "NES",
        "NEW",
        "NF",
        "NI",
        "NK",
        "NL",
        "NM",
        "NMS",
        "NOH",
        "NOM",
        "NR",
        "NU",
        "NVP",
        "NW",
        "NWM",
        "OA",
        "OAL",
        "OB",
        "OD",
        "OE",
        "OF",
        "OG",
        "OH",
        "OHA",
        "OHV",
        "OHZ",
        "OL",
        "OPR",
        "OS",
        "OSL",
        "OVP",
        "P",
        "PA",
        "PAF",
        "PAN",
        "PB",
        "PCH",
        "PE",
        "PF",
        "PI",
        "PIR",
        "PLÖ",
        "PM",
        "PR",
        "PS",
        "R",
        "RA",
        "RD",
        "RE",
        "REG",
        "RO",
        "ROS",
        "ROW",
        "RP",
        "RPL",
        "RS",
        "RT",
        "RÜD",
        "RÜG",
        "RV",
        "RW",
        "RZ",
        "S",
        "SAD",
        "SAL",
        "SAW",
        "SB",
        "SC",
        "SDL",
        "SE",
        "SG",
        "SH",
        "SHA",
        "SHG",
        "SHK",
        "SHL",
        "SI",
        "SIG",
        "SIM",
        "SK",
        "SL",
        "SLF",
        "SLK",
        "SLS",
        "SM",
        "SN",
        "SO",
        "SOK",
        "SÖM",
        "SON",
        "SP",
        "SPN",
        "SR",
        "ST",
        "STA",
        "STD",
        "SU",
        "SÜW",
        "SW",
        "SZ",
        "TDO",
        "TBB",
        "TF",
        "TG",
        "THL",
        "THW",
        "TIR",
        "TÖL",
        "TR",
        "TS",
        "TÜ",
        "TUT",
        "UE",
        "UL",
        "UM",
        "UN",
        "V",
        "VB",
        "VEC",
        "VER",
        "VIE",
        "VK",
        "VR",
        "VS",
        "W",
        "WAF",
        "WAK",
        "WB",
        "WE",
        "WEN",
        "WES",
        "WF",
        "WHV",
        "WI",
        "WIL",
        "WL",
        "WM",
        "WN",
        "WND",
        "WO",
        "WOB",
        "WST",
        "WT",
        "WTM",
        "WÜ",
        "WUG",
        "WUN",
        "WW",
        "WZ",
        "Y",
        "Z",
        "ZW",
    )

    license_plate_suffix = (
        "-??-%@@@",
        "-?-%@@@",
    )

    def license_plate(self) -> str:
        """Generate a license plate."""
        prefix: str = self.random_element(self.license_plate_prefix)
        suffix = self.bothify(
            self.random_element(self.license_plate_suffix),
            letters=string.ascii_uppercase,
        )
        return prefix + suffix



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """
    Implement automotive provider for `zh_CN` locale.
    electric vehicles or downtown-restricted plates are not included
    """

    province_code = (
        "京",
        "津",
        "冀",
        "晋",
        "蒙",
        "辽",
        "吉",
        "黑",
        "沪",
        "苏",
        "浙",
        "皖",
        "闽",
        "赣",
        "鲁",
        "豫",
        "鄂",
        "湘",
        "粤",
        "桂",
        "琼",
        "渝",
        "川",
        "贵",
        "云",
        "藏",
        "陕",
        "甘",
        "青",
        "宁",
        "新",
    )

    def license_plate(self) -> str:
        """Generate a license plate."""
        pattern: str = str(self.random_element(self.province_code)) + self.random_uppercase_letter() + "-#####"
        return self.numerify(self.generator.parse(pattern))



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``zh_TW`` locale.

    Sources:
    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Taiwan

    """

    license_formats = (
        "####-??",
        "??-####",
        # Commercial vehicles since 2012
        "???-###",
        # New format since 2014
        "???-####",
    )



from collections import OrderedDict

from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = OrderedDict(
        (
            ("{{first_name_female}} {{last_name_female}}", 0.97),
            ("{{prefix_female}} {{first_name_female}} {{last_name_female}}", 0.015),
            ("{{first_name_female}} {{last_name_female}} {{suffix}}", 0.02),
            (
                "{{prefix_female}} {{first_name_female}} {{last_name_female}} {{suffix}}",
                0.005,
            ),
        )
    )

    formats_male = OrderedDict(
        (
            ("{{first_name_male}} {{last_name_male}}", 0.97),
            ("{{prefix_male}} {{first_name_male}} {{last_name_male}}", 0.015),
            ("{{first_name_male}} {{last_name_male}} {{suffix}}", 0.02),
            (
                "{{prefix_male}} {{first_name_male}} {{last_name_male}} {{suffix}}",
                0.005,
            ),
        )
    )

    formats = formats_male.copy()
    formats.update(formats_female)

    first_names_male = (
        "Adam",
        "Alexander",
        "Alexandr",
        "Aleš",
        "Alois",
        "Antonín",
        "Arnošt",
        "Bedřich",
        "Bohumil",
        "Bohumír",
        "Bohuslav",
        "Břetislav",
        "Dalibor",
        "Daniel",
        "David",
        "Denis",
        "Dominik",
        "Dušan",
        "Eduard",
        "Emil",
        "Erik",
        "Filip",
        "František",
        "Hynek",
        "Igor",
        "Ivan",
        "Ivo",
        "Jakub",
        "Jan",
        "Jaromír",
        "Jaroslav",
        "Jindřich",
        "Jiří",
        "Josef",
        "Jozef",
        "Ján",
        "Kamil",
        "Karel",
        "Kryštof",
        "Ladislav",
        "Leoš",
        "Libor",
        "Lubomír",
        "Luboš",
        "Ludvík",
        "Luděk",
        "Lukáš",
        "Marcel",
        "Marek",
        "Marian",
        "Martin",
        "Matyáš",
        "Matěj",
        "Michael",
        "Michal",
        "Milan",
        "Miloslav",
        "Miloš",
        "Miroslav",
        "Oldřich",
        "Ondřej",
        "Otakar",
        "Patrik",
        "Pavel",
        "Peter",
        "Petr",
        "Přemysl",
        "Radek",
        "Radim",
        "Radomír",
        "Radovan",
        "René",
        "Richard",
        "Robert",
        "Robin",
        "Roman",
        "Rostislav",
        "Rudolf",
        "Samuel",
        "Stanislav",
        "Tadeáš",
        "Tomáš",
        "Vasyl",
        "Viktor",
        "Vilém",
        "Vladimír",
        "Vladislav",
        "Vlastimil",
        "Vojtěch",
        "Vratislav",
        "Václav",
        "Vít",
        "Vítězslav",
        "Zbyněk",
        "Zdeněk",
        "Šimon",
        "Štefan",
        "Štěpán",
    )

    first_names_female = (
        "Adéla",
        "Alena",
        "Alexandra",
        "Alice",
        "Alžběta",
        "Andrea",
        "Aneta",
        "Anežka",
        "Anna",
        "Barbora",
        "Blanka",
        "Blažena",
        "Bohumila",
        "Božena",
        "Dagmar",
        "Dana",
        "Daniela",
        "Danuše",
        "Denisa",
        "Dominika",
        "Drahomíra",
        "Eliška",
        "Emilie",
        "Eva",
        "Františka",
        "Gabriela",
        "Hana",
        "Helena",
        "Ilona",
        "Irena",
        "Iva",
        "Ivana",
        "Iveta",
        "Jana",
        "Jarmila",
        "Jaroslava",
        "Jindřiška",
        "Jitka",
        "Jiřina",
        "Julie",
        "Kamila",
        "Karolína",
        "Kateřina",
        "Klára",
        "Kristina",
        "Kristýna",
        "Květa",
        "Květoslava",
        "Ladislava",
        "Lenka",
        "Libuše",
        "Lucie",
        "Ludmila",
        "Magdalena",
        "Magdaléna",
        "Marcela",
        "Marie",
        "Markéta",
        "Marta",
        "Martina",
        "Michaela",
        "Milada",
        "Milena",
        "Miloslava",
        "Miluše",
        "Miroslava",
        "Monika",
        "Mária",
        "Naděžda",
        "Natálie",
        "Nela",
        "Nikol",
        "Nikola",
        "Olga",
        "Pavla",
        "Pavlína",
        "Petra",
        "Radka",
        "Renata",
        "Renáta",
        "Romana",
        "Růžena",
        "Sabina",
        "Simona",
        "Soňa",
        "Stanislava",
        "Sára",
        "Tereza",
        "Vendula",
        "Veronika",
        "Viktorie",
        "Vladimíra",
        "Vlasta",
        "Věra",
        "Zdenka",
        "Zdeňka",
        "Zuzana",
        "Štěpánka",
        "Šárka",
        "Žaneta",
    )

    first_names = first_names_male + first_names_female

    last_names_male = (
        "Bartoš",
        "Beneš",
        "Blažek",
        "Bláha",
        "Doležal",
        "Dušek",
        "Dvořák",
        "Fiala",
        "Holub",
        "Horák",
        "Hájek",
        "Jelínek",
        "Kadlec",
        "Kolář",
        "Kopecký",
        "Kratochvíl",
        "Krejčí",
        "Král",
        "Kučera",
        "Kříž",
        "Malý",
        "Marek",
        "Mareš",
        "Mašek",
        "Moravec",
        "Novotný",
        "Novák",
        "Němec",
        "Pokorný",
        "Polák",
        "Pospíšil",
        "Procházka",
        "Růžička",
        "Sedláček",
        "Soukup",
        "Svoboda",
        "Urban",
        "Vaněk",
        "Veselý",
        "Vlček",
        "Zeman",
        "Čermák",
        "Černý",
        "Říha",
        "Šimek",
        "Štěpánek",
        "Šťastný",
    )

    last_names_female = (
        "Bartošová",
        "Benešová",
        "Beranová",
        "Blažková",
        "Bláhová",
        "Doležalová",
        "Dušková",
        "Dvořáková",
        "Fialová",
        "Holubová",
        "Horáková",
        "Hájková",
        "Jandová",
        "Jelínková",
        "Kadlecová",
        "Kolářová",
        "Kopecká",
        "Kratochvílová",
        "Krejčová",
        "Králová",
        "Kučerová",
        "Křížová",
        "Machová",
        "Malá",
        "Marešová",
        "Marková",
        "Mašková",
        "Moravcová",
        "Novotná",
        "Nováková",
        "Němcová",
        "Pokorná",
        "Poláková",
        "Pospíšilová",
        "Procházková",
        "Růžičková",
        "Sedláčková",
        "Soukupová",
        "Svobodová",
        "Tichá",
        "Urbanová",
        "Vacková",
        "Vaňková",
        "Veselá",
        "Vlčková",
        "Vávrová",
        "Zemanová",
        "Čermáková",
        "Černá",
        "Říhová",
        "Šimková",
        "Štěpánková",
        "Šťastná",
    )

    last_names = last_names_male + last_names_female

    degrees = ("JUDr.", "Ing.", "Bc.", "Mgr.", "MUDr.", "RNDr.")

    prefixes_male = ("pan",) + degrees

    prefixes_female = ("paní", "slečna") + degrees

    suffixes = ("CSc.", "DiS.", "Ph.D.", "Th.D.")
